#ifndef _ADF7030_1__MEM_H_
#define _ADF7030_1__MEM_H_

#include <drivers/spi/adi_spi.h>
#include <devices/rf/adf703x/adi_adf7030-1_reg.h>

/** \addtogroup adf7030-1 adf7030-1 Driver
 *  @{
 */

/** \addtogroup adf7030-1__mem PHY Memory Interface
 *  @{
 */

extern uint32_t ADF7030_1_SPI_PNTR[8];


//Forward Typedef declarartion
typedef struct MEM_DESCRIPTOR_Type              MEM_DESCRIPTOR_Type;
typedef struct MEM_CFG_DESCRIPTOR_Type          MEM_CFG_DESCRIPTOR_Type;

typedef struct{
    uint8_t OffsetLen;          ///Size of the Offset list
    uint8_t * pOffset;          ///Pointer to the first offset
}RANDOM_XFER_DESCRIPTOR_Type;

typedef struct{
    uint16_t Size;              ///Byte size of the Block to transfer
    uint16_t Offset;            ///Byte offset relative to the base location
}BLOCK_XFER_DESCRIPTOR_Type;

typedef struct{
    uint16_t Size;              ///Byte size of the Block to transfer
    const uint8_t unused;
    const uint8_t WordXfer;     ///If !NULL, force 32bits SPI transfer
    const uint32_t Addr;        ///Address of the memory bloc in the PHY memory space
    const uint8_t *pData;       ///pointer to Data block
}DATA_BLOCK_DESCRIPTOR_Type;

/**
* \todo Define a config structure for revision tracking/conpatibility
*/
typedef struct {
    char config_name[16];
    uint16_t build_number;
    char family_str[4];
    uint16_t adiid;
    uint16_t chipid;
    uint8_t full_layer_num;
    uint8_t eco_num;
    uint16_t svn_version;
    char unused[66];
} config_info_t;

/** Generic Type to describes the Radio PHY shadow memory structures.
    The application layer is responsible for allocating buffer.
*/
struct MEM_DESCRIPTOR_Type{
    char		        Name[16];	///String with name of memory bloc
    uint16_t 	                Size;		///size of the memory bloc
    uint8_t 	                ByteMode;	///0->8bits, 1->32bits SPI transfert mode, 2->8bits Only, 3->32bits only
    uint8_t                     XferMode;       ///0->Block, 1->Random SPI transfer mode
    uint32_t 	                PhyAddr;	///address of the memory bloc in the PHY memory space
    void *		        pBuff;		///pointer to the start of the shadow memory bloc in the host memory space
    void *                      pBuffRef;       ///pointer to the initial value of the shadow memory bloc in the host memory space
    BLOCK_XFER_DESCRIPTOR_Type  BlockXfer;      ///descriptor for block access
    RANDOM_XFER_DESCRIPTOR_Type RandXfer;       ///descriptor for random access
    MEM_DESCRIPTOR_Type *       pNextBloc;      ///If !NULL, points to the next MEM_DESCRIPTOR_Type block to be chained
};

/** Generic Type to describes the Radio PHY configuration memory structures.
    Data is provided by the setting calculator
*/
struct MEM_CFG_DESCRIPTOR_Type{
    char		        Name[16];	///String with name of configuration sequence
    uint16_t 	                Size;		///Total size of the configuration
    uint16_t                    nbBlock;        ///Number of data blocks
    config_info_t *             pInfo;          ///Pointer to the configuration info structure in the host memory
    MEM_CFG_DESCRIPTOR_Type *   pNext;          ///Pointer to the next configuration sequence if desired, else NULL
    DATA_BLOCK_DESCRIPTOR_Type  BLOCKS[];
};


#define PHY_MEM_READ true
#define PHY_MEM_WRITE false


/* Write Configuration data to Radio PHY */
uint32_t adf7030_1__WriteConfig(
    ADI_SPI_HANDLE hSPIDevice,                              
    MEM_CFG_DESCRIPTOR_Type * pCONFIG                                
);

/* Read Configuration data from Radio PHY */
uint32_t adf7030_1__ReadConfig(
    ADI_SPI_HANDLE hSPIDevice,                              
    MEM_CFG_DESCRIPTOR_Type * pCONFIG                                
);

/* Verify Configuration integrity */
uint32_t adf7030_1__VerifyConfig(
    ADI_SPI_HANDLE hSPIDevice,                              
    MEM_CFG_DESCRIPTOR_Type * pCONFIG                                
);
  
/* Transfers Configuration between Host and the radio PHY */
uint32_t adf7030_1__XferConfig(
    ADI_SPI_HANDLE hSPIDevice,
    MEM_CFG_DESCRIPTOR_Type * pCONFIG,
    bool READ_nWRITE,
    bool CHECK_ONLY
);

/* Transfers generic memory blocks between Host shadow and the radio PHY */
uint32_t adf7030_1__SPI_Xfer_Mem(
    ADI_SPI_HANDLE hSPIDevice,
    MEM_DESCRIPTOR_Type * pMEM,
    bool READ_nWRITE
);

/* Read 32bits memory location via the SPI */
void adf7030_1__SPI_SetMem32(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t Value
);

/* Read 32bits memory location via the SPI */
uint32_t adf7030_1__SPI_GetMem32(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr
);

/* Read 24bits memory location via the SPI */
void adf7030_1__SPI_SetMem24(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t Value
);

/* Read 24bits memory location via the SPI */
uint32_t adf7030_1__SPI_GetMem24(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr
);

/* Read 16bits memory location via the SPI */
void adf7030_1__SPI_SetMem16(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint16_t Value
);

/* Read 16bits memory location via the SPI */
uint16_t adf7030_1__SPI_GetMem16(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr
);

/* Read 8bits memory location via the SPI */
void adf7030_1__SPI_SetMem8(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint8_t Value
);

/* Read 8bits memory location via the SPI */
uint8_t adf7030_1__SPI_GetMem8(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr
);

/* Generic small memory write (up to 4 bytes) via the SPI */
void adf7030_1__SPI_SetBytes(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t Value,
    uint32_t nbBytes
);

/* Generic small memory read (up to 4 bytes) via the SPI */
uint32_t adf7030_1__SPI_GetBytes(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t nbBytes
);

/* Generic function to Set Field value via SPI */
void adf7030_1__SPI_SetField(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t Pos,
    uint32_t Size,
    uint32_t Val
);

/* Generic Function to Get Field value via SPI */
uint32_t adf7030_1__SPI_GetField(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t Pos,
    uint32_t Size
);

/* Generic Macro for setting a bitfield to a value (uses adf7030_hw_macro.h & adf7030_fw_macro.h definitions) */
#define adf7030_1__WRITE_FIELD(fName, fValue) adf7030_1__SPI_SetField( hSPIDevice, \
                                                                       fName##_Addr, \
                                                                       fName##_Pos, \
                                                                       fName##_Size, \
                                                                       fValue )

/* Alternative version of "adf7030_1__SET_FIELD()" macro calling directly adf7030_1__SPI_SetBytes() function */
#define adf7030_1__WRITE_FIELD2(fName, fValue) \
{ \
    /* Compute Bitfields parameters */ \
    uint32_t fStartByte = (fName##_Pos >> 3); \
    uint32_t fPos_fromStartByte = fName##_Pos - (fStartByte << 3); \
    uint32_t fLenBytes = ((fName##Size - 1) >> 3) + 1; \
 \
    /* Read Bitfield vis SPI */ \
    uint32_t fVal = adf7030_1__SPI_GetBytes( hSPIDevice, Addr + fStartByte, fLenBytes); \
 \
    /* Compute the Mask */ \
    uint32_t fMsk = ((1 << fName##_Size) - 1) << fPos_fromStartByte; \
 \
    /* Bound Val to the Mask */ \
    fValue <<= fPos_fromStartByte; \
    fValue &= fMsk; \
 \
    /* And SPI returned value with inverted Mask */ \
    fVal &= (~fMsk); \
 \
    /* Or SPI returned value with fValue */ \
    fVal |= fValue; \
 \
    /* Write back Bitfield vis SPI */ \
    adf7030_1__SPI_SetBytes( hSPIDevice, Addr + fStartByte, fValue, fLenBytes); \
};


/* Generic Macro for reading a bitfield value (uses adf7030_hw_macro.h & adf7030_fw_macro.h definitions) */
#define adf7030_1__READ_FIELD(fName) adf7030_1__SPI_GetField( hSPIDevice, \
                                                              fName##_Addr, \
                                                              fName##_Pos, \
                                                              fName##_Size )

/* Alternative version of "adf7030_1__GET_FIELD()" macro calling directly adf7030_1__SPI_SetBytes() & adf7030_1__SPI_SetBytes() functions */
#define adf7030_1__READ_FIELD2(fName) \
({ \
    uint32_t fVal; \
 \
    /* Compute Bitfields parameters */ \
    uint32_t fStartByte = (fName##_Pos >> 3); \
    uint32_t fPos_fromStartByte = fName##_Pos - (fStartByte << 3); \
    uint32_t fLenBytes = ((fName##Size - 1) >> 3) + 1; \
 \
    /* Read Bitfield vis SPI */ \
    fVal = adf7030_1__SPI_GetBytes( hSPIDevice, Addr + fStartByte, fLenBytes); \
 \
    /* Compute the Mask */ \
    uint32_t fMsk = ((1 << fName##_Size) - 1) << fPos_fromStartByte; \
 \
    /* And SPI returned value with Mask */ \
    fVal &= fMsk; \
 \
    /* Shift right by fPos_fromStartByte */ \
    fVal >>= fPos_fromStartByte; \
 \
    fVal; \
});


/* Generic Macro for setting a register address to a value (uses adf7030_hw_macro.h & adf7030_fw_macro.h definitions) */
#define adf7030_1__WRITE_REG(rName, rValue) adf7030_1__SPI_SetBytes( hSPIDevice, \
                                                                     rName##_Addr, \
                                                                     rValue, \
                                                                     4 )

/* Generic Macro for setting a register address to a value (uses adf7030_hw_macro.h & adf7030_fw_macro.h definitions) */
#define adf7030_1__READ_REG(rName) adf7030_1__SPI_GetBytes( hSPIDevice, \
                                                            rName##_Addr, \
                                                            4 )


/** @} */ /* End of group adf7030-1__mem PHY Memory Interface */
/** @} */ /* End of group adf7030-1 adf7030-1 Driver */

#endif /* _ADF7030_1__SPI_H_ */