#ifndef _ADF7030_1__SPI_H_
#define _ADF7030_1__SPI_H_

#include <drivers/spi/adi_spi.h>
#include <devices/rf/adf703x/adi_adf7030-1_reg.h>


/** \addtogroup adf7030-1 adf7030-1 Driver
 *  @{
 */

/** \addtogroup adf7030-1__spi SPI Interface layer
 *  @{
 */


/** define size of data buffers, DMA max size is 255 */
/** ( 8bytes max used for command framing in word transfert )
 *           32bits trx = 62
 *            8bits trx = 252
 */
#define ADF7030_1_SPI_BUFFER_SIZE 256u
#define ADF7030_1_SPI_MAX_TRX_SIZE (ADF7030_1_SPI_BUFFER_SIZE - 8u)
//#define ADF7030_1_SPI_MAX_TRX_SIZE 32u

/* SPI instance and speed */
#define SPI_DEV_NUM                         2
#define SPI_DEV_BITRATE                     3000000

/* CS port to which the CSN line of adf7030-1 is connected */ 
#define SPI_CS_NUM                          ADI_SPI2_CS3

/* GPIO port and pin to which default interrupt pin1 of adf7030-1 is connected */
//P3-3  --> P0_01/GPIO01/SPI0_MOSI/SPT0_BFS or P0_15/GPIO15/SYS_WAKE0
//P2-10 --> P2_04/GPIO36/ADC0_VIN1
#define ADF703x_INT1_GPIO_PORT              ADI_GPIO_PORT0
#define ADF703x_INT1_GPIO_PIN               ADI_GPIO_PIN_15

/* GPIO port and pin to which default interrupt pin2 of adf7030-1 is connected */
//P3-20 --> P1_00/GPIO16/SYS_WAKE1 or P2_08/GPIO40/ADC0_VIN5/SPI0_CS2
#define ADF703x_INT2_GPIO_PORT              ADI_GPIO_PORT1
#define ADF703x_INT2_GPIO_PIN               ADI_GPIO_PIN_0

extern uint32_t ADF7030_1_SPI_PNTR[8];
   

/*! Enumeration of different SPI pointer type*/
typedef enum {
    PNTR_SETUP_ADDR     = 0,
    PNTR_SRAM_ADDR      = 1,
    PNTR_MCR_LOW        = 2,
    PNTR_MCR_HIGH       = 3,
    PNTR_IRQ_CTRL_ADDR  = 4,
    PNTR_CUSTOM0_ADDR   = 5,    //Used for generic byte Access
    PNTR_CUSTOM1_ADDR   = 6,    //Used for TX
    PNTR_CUSTOM2_ADDR   = 7 
} ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE;

/*! Enumeration of Host SPI transaction type*/
typedef enum {
    SPI_WRITE                   = 0,
    SPI_READ                    = 1
} ADI_ADF7030_1_RADIO_SPI_TRX_TYPE;

    
typedef struct{
  union {
    uint8_t  VALUE;                          /*!< SPI Slave Status                                                      */
  
    struct {
      uint8_t  MEM_ACC_STATUS:  1;            /*!< *Instantaneous* value of an AHB read data underflow condition
                                                    for memory reads. The same bit doubles up as an AHB bus error
                                                     indicator for memory writes.                                         */
      uint8_t  FW_STATUS  :  4;               /*!< *Instantaneous* value of the firmware status of the radio controller
                                                    state machine.                                                        */
      uint8_t  CMD_READY  :  1;               /*!< *Instantaneous* value of the indicator to the host that no posted
                                                    (to the SPI Slave) radio controller command is currently awaiting
                                                     servicing by the firmware radio state machine running on the
                                                     Cortex-M0.                                                           */
      uint8_t  EXT_IRQ_STATUS:  1;            /*!< *Instantaneous* value of the commoned-up (bitwise OR of the)
                                                    external interrupt outputs from the ADF7030, made available
                                                     in this format to the host for use in pin-limited systems.           */
      uint8_t  SPIS_READY :  1;               /*!< *Instantaneous* value of the confirmation to the external host
                                                    that the HCLK clock domain in the ADF7030 has power, is out
                                                     of reset and has a running clock.                                    */
    };                                     /*!< BitSize                                                               */ 
  };
} ADI_ADF7030_1_RADIO_SPI_STATUS_TYPE;


/*! \cond PRIVATE */ 

/* memory command */
#define ADF703x_SPI_MEM_WRITE       (0 << 6)
#define ADF703x_SPI_MEM_READ        (1 << 6)
#define ADF703x_SPI_MEM_RANDOM      (0 << 5)
#define ADF703x_SPI_MEM_BLOCK       (1 << 5)
#define ADF703x_SPI_MEM_POINTER     (0 << 4)
#define ADF703x_SPI_MEM_ADDRESS     (1 << 4)
#define ADF703x_SPI_MEM_SHORT       (0 << 3)
#define ADF703x_SPI_MEM_LONG        (1 << 3)

extern void thumb_aligned_memcpy( void* dest, const void* src, uint32_t bytesToCopy );
extern void thumb_aligned_memset( void* dest, const uint32_t val, uint32_t bytesToCopy );

/*! \endcond */

uint32_t adf7030_1__SPI_Init(
    void
);

uint32_t adf7030_1__SPI_UnInit(
    void
);

uint32_t adf7030_1__SPI_SetSpeed(
    const uint32_t RateHz
);

uint32_t adf7030_1__SPI_SetMMapPointers(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t * pPNTR
);

uint32_t adf7030_1__SPI_GetMMapPointers(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t * pPNTR
);

void adf7030_1__SPI_FindMMapPointer(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE * pPNTR_ID,
    int32_t * ByteOffset
);
    
uint32_t adf7030_1__SPI_SetMMapCustomPntr0(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr
);

uint32_t adf7030_1__SPI_wr_word_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint32_t *pDataIn
);

uint32_t adf7030_1__SPI_rd_word_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint32_t * pDataOut
);

uint32_t adf7030_1__SPI_rd_cmp_word_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint32_t * pDataOut,
    uint32_t * pDataRef
);

uint32_t adf7030_1__SPI_wr_word_b_p(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint32_t * pDataIn
);

uint32_t adf7030_1__SPI_rd_word_b_p(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint32_t * pDataOut
);

uint32_t adf7030_1__SPI_rd_cmp_word_b_p(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint32_t * pDataOut,
    uint32_t * pDataRef
);

uint32_t adf7030_1__SPI_wr_byte_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint8_t * pDataIn
);

uint32_t adf7030_1__SPI_rd_byte_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint8_t * pDataOut
);

uint32_t adf7030_1__SPI_rd_cmp_byte_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint8_t * pDataOut,
    uint8_t * pDataRef
);
    
uint32_t adf7030_1__SPI_wr_byte_p_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint8_t * pDataIn
);
    
uint32_t adf7030_1__SPI_rd_byte_p_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint8_t * pDataOut
);

uint32_t adf7030_1__SPI_rd_cmp_byte_p_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint8_t * pDataOut,
    uint8_t * pDataRef
);

uint32_t adf7030_1__SPI_Block_Xfer(
    ADI_SPI_HANDLE                      hSPIDevice,
    uint32_t                            cmdOffset,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE   PNTR_ID,
    uint32_t                            Addr,
    uint32_t                            nbBytes,
    void *                              pDataIO,
    void *                              pDataRef
);

uint32_t adf7030_1__SPI_word_block_transaction(
    ADI_SPI_HANDLE *hSPIDevice,
    uint32_t offset,
    uint32_t nb_bytes,
    uint32_t * pDataIO,
    uint32_t * pDataRef
);

uint32_t adf7030_1__SPI_byte_block_transaction(
    ADI_SPI_HANDLE *hSPIDevice,
    uint32_t offset,
    uint32_t nb_bytes,
    uint8_t * pDataIO,
    uint8_t * pDataRef
);
      
uint32_t adf7030_1__SPI_word_random_transaction(
    ADI_SPI_HANDLE *hSPIDevice,
    uint32_t offset,
    uint32_t nb_word,
    uint32_t * pAddrIO,
    uint32_t * pDataIO,
    bool pDataIO_is_pointer
);

uint32_t adf7030_1__SPI_byte_random_transaction(
    ADI_SPI_HANDLE *hSPIDevice,
    uint32_t offset,
    uint32_t nb_word,
    uint8_t * pAddrIO,
    uint8_t * pDataIO
);

/** @} */ /* End of group adf7030-1__spi SPI Interface layer */
/** @} */ /* End of group adf7030-1 adf7030-1 Driver */

#endif /* _ADF7030_1__SPI_H_ */