/*********************************************************************************

Copyright(c) 2016 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file     adf7030-1_mem.c
*
* @brief    adf7030-1 Memory Accress Functions.
*               - SPI communication with between Host and Radio.
*               - Shadow Memory MAP
*
* @details  ...
*           ...
*           ...
*/

/** \addtogroup adf7030-1 adf7030-1 Driver
 *  @{
 */

/** \addtogroup adf7030-1__mem PHY Memory Interface
 *  @{
 */
#ifndef _ADF7030_1__SPI_C_
/*! \cond PRIVATE */
#define _ADF7030_1__SPI_C_

#include <stddef.h>
#include <string.h>
#include <assert.h>
#include "system.h"
#include "common.h"

#include <drivers/spi/adi_spi.h>
//#include <devices/rf/adf703x/adi_adf7030-1.h>
//#include <devices/rf/adf703x/adi_adf7030-1_reg.h>
#include <devices/rf/adf703x/adf7030-1__spi.h>
#include <devices/rf/adf703x/adf7030-1__mem.h>


#ifdef __ICCARM__
/*
* IAR MISRA C 2004 error suppressions.
*
* Pm140 (rule 11.3): a cast should not be performed between a pointer type and an integral type
*   The rule makes an exception for memory-mapped register accesses.
*
* Pm073 (rule 14.7): a function should have a single point of exit.
* Pm143 (rule 14.7): a function should have a single point of exit at the end of the function.
*   Multiple returns are used for error handling.
*/
#pragma diag_suppress=Pm073,Pm143
#endif /* __ICCARM__ */

/*! \endcond */

/**
 * @brief       Function call to write Configuration to Radio PHY
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pCONFIG         Pointer to Host configuration structure.
 *
 * @note                        Configuration data is provided by ADI RF Setting Calculator.
 *                              @sa USER_CONFIG_20.h for an example.
 *
 * @return      Status
 *  - #SUCCESS                  If the Configuration was written successfully in the Host.
 *  - #FAILURE                  [D] If the patch transfert failed.
 */

uint32_t adf7030_1__WriteConfig(
    ADI_SPI_HANDLE hSPIDevice,                              
    MEM_CFG_DESCRIPTOR_Type * pCONFIG                               
)
{
    return(adf7030_1__XferConfig( hSPIDevice, pCONFIG, PHY_MEM_WRITE, false));
}

/**
 * @brief       Function call to read Configuration from Radio PHY
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pCONFIG         Pointer to Host configuration structure.
 *
 * @note                        Configuration data is provided by ADI RF Setting Calculator.
 *                              @sa USER_CONFIG_20.h for an example.
 *
 * @return      Status
 *  - #SUCCESS                  If the Configuration was readback successfully in the Host.
 *  - #FAILURE                  [D] If the Configuration transfert failed.
 */

uint32_t adf7030_1__ReadConfig(
    ADI_SPI_HANDLE hSPIDevice,                              
    MEM_CFG_DESCRIPTOR_Type * pCONFIG                               
)
{
    return(adf7030_1__XferConfig( hSPIDevice, pCONFIG, PHY_MEM_READ, false));
}

/**
 * @brief       Function call to verify the integrity of the patch
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pCONFIG         Pointer to Host configuration structure.
 *
 * @note                        This function only disable the patch, it can be 
 *                              re-enable at a later stage by calling the 
 *                              adf7030_1__InjectPatch() function.
 *                              This function modifies ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR].
 *
 * @return      Status
 *  - #SUCCESS                  If the Configuration in the PHY matches the reference in the Host.
 *  - #FAILURE                  [D] If the Configuration is different.
 */

uint32_t adf7030_1__VerifyConfig(
    ADI_SPI_HANDLE hSPIDevice,                              
    MEM_CFG_DESCRIPTOR_Type * pCONFIG                                
)
{
  return(adf7030_1__XferConfig( hSPIDevice, pCONFIG, PHY_MEM_READ, true));
}

/**
 * @brief       Generic Configuration transfers function between Host and the Radio PHY
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pCONFIG         Pointer to Host configuration structure.
 *
 * @param [in]  READ_nWRITE     Boolean  -TRUE to write Config from Host to the Radio PHY
 *                                       -FALSE to read Config from Radio PHY back to Host
 *
 * @param [in]  CHECK_ONLY      Boolean  -TRUE to check PHY Config versus Host Reference
 *                                       -FALSE [D] to perform READ_nWRITE operation
 *
 * @note                        This function can modify ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR].
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__XferConfig(
    ADI_SPI_HANDLE hSPIDevice,
    MEM_CFG_DESCRIPTOR_Type * pCONFIG,
    bool READ_nWRITE,
    bool CHECK_ONLY
)
{
    bool exit;
    do{
  
        for(uint32_t i = 0; i < pCONFIG->nbBlock; i++)
        {
            /* Get Memory Block Address */   
            uint32_t PHYAddr = pCONFIG->BLOCKS[i].Addr;
                  
            /* 8bits Mode SPI transfers mode by default */
            if(pCONFIG->BLOCKS[i].WordXfer == 0)
            {
                ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE PNTR_id;
                int32_t PNTR_offset;
            
                /* Get good PNTRid from ADF7030_1_SPI_PNTR[] */
                adf7030_1__SPI_FindMMapPointer( hSPIDevice, PHYAddr, &PNTR_id, &PNTR_offset);
                
                /* Setup pointer to Host start of data */
                uint8_t * pHostMem;
                pHostMem = (uint8_t *)pCONFIG->BLOCKS[i].pData;
                
                uint32_t size = pCONFIG->BLOCKS[i].Size;
    
                /* No valid PHY SPI pointer available, use 8bits access with no offset */
                if(PNTR_id == 0)
                {
                    /* Change the SPI custom pointer 0 */
                    if(adf7030_1__SPI_SetMMapCustomPntr0( hSPIDevice, PHYAddr ) != SUCCESS)
                    {
                        return FAILURE;
                    }
                    
                    if(CHECK_ONLY)
                    {
                        /* Read from radio PHY and compare to host */
                        if(adf7030_1__SPI_rd_cmp_byte_p_a( hSPIDevice, PNTR_CUSTOM0_ADDR, size, NULL, pHostMem) != SUCCESS)
                        {
                            return FAILURE;
                        } 
                    }
                    else
                    {
                        /* Copy Host MEM memory block to PHY memory block via the SPI */
                        if(READ_nWRITE == true)
                        {
                            /* Read from radio PHY */
                            if(adf7030_1__SPI_rd_byte_p_a( hSPIDevice, PNTR_CUSTOM0_ADDR, size, pHostMem) != SUCCESS)
                            {
                                return FAILURE;
                            }                    
                        }else{
                            /* Write to radio PHY */
                            if(adf7030_1__SPI_wr_byte_p_a( hSPIDevice, PNTR_CUSTOM0_ADDR, size, pHostMem) != SUCCESS)
                            {
                                return FAILURE;
                            }  
                        }
                    }
                }
                /* Valid PHY SPI pointer available, use 8bits access with pointer offset */
                else
                {
                    if(CHECK_ONLY)
                    {
                        /* Read from radio PHY and compare to host */
                        if(adf7030_1__SPI_rd_cmp_byte_b_a( hSPIDevice, PNTR_id, PNTR_offset, size, NULL, pHostMem) != SUCCESS)
                        {
                            return FAILURE;
                        } 
                    }
                    else
                    {
                      /* Copy Host MEM memory block to PHY memory block via the SPI */
                        if(READ_nWRITE == true)
                        {
                            /* Read from radio PHY */
                            if(adf7030_1__SPI_rd_byte_b_a( hSPIDevice, PNTR_id, PNTR_offset, size, pHostMem) != SUCCESS)
                            {
                                return FAILURE;
                            }                    
                        }else{
                            /* Write to radio PHY */
                            if(adf7030_1__SPI_wr_byte_b_a( hSPIDevice, PNTR_id, PNTR_offset, size, pHostMem) != SUCCESS)
                            {
                                return FAILURE;
                            }  
                        }
                    }
                }
            }
            /* This block only support 32bits Mode SPI transfers, Let us just use block modes */
            else
            {
                /* Setup pointer to Host start of data */
                uint32_t * pHostMem;
                pHostMem = (uint32_t *)pCONFIG->BLOCKS[i].pData;
                
                uint32_t size = pCONFIG->BLOCKS[i].Size >> 2;
                
                if(CHECK_ONLY)
                {
                    /* Read from radio PHY and compare to host */
                    if(adf7030_1__SPI_rd_cmp_word_b_a( hSPIDevice, PHYAddr, size, NULL, pHostMem) != SUCCESS)
                    {
                        return FAILURE;
                    }   
                }
                else
                {
                      /* Copy Host MEM memory block to PHY memory block via the SPI */
                    if(READ_nWRITE == true)
                    {
                        /* Read from radio PHY */
                        if(adf7030_1__SPI_rd_word_b_a( hSPIDevice, PHYAddr, size, pHostMem) != SUCCESS)
                        {
                            return FAILURE;
                        }                    
                    }else{
                        /* Write to radio PHY */
                        if(adf7030_1__SPI_wr_word_b_a( hSPIDevice, PHYAddr, size, pHostMem) != SUCCESS)
                        {
                            return FAILURE;
                        }  
                    } 
                }
            }
        }  
    
        /* Check if nested MEM Configuration transfert is present */
        if(pCONFIG->pNext)
        {
            /* Move pointer to MEM structure to the following one */
            pCONFIG = pCONFIG->pNext;
            exit = false;
        }
        else
        {
            exit = true;
        }
    }while(exit == false);
    
    return SUCCESS;
}

/**
 * @brief       Generic memory transfers function between Host shadow and the Radio PHY
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pMEM            Descriptor to the paired Host/PHY memory region.
 *
 * @param [in]  READ_nWRITE     Boolean  -TRUE to write from Host shadow to the Radio PHY
 *                                       -FALSE to read from Radio PHY back to Host shadow
 *
 * @note                        This function assume the following:
 *                              - Both Host shadow block (&(pMEM->pBuff)) and 
 *                              PHY memory blocks (pMEM->PhyAddr) are starting at 
 *                              a word alligned location.
 *                              - ADF7030_1_SPI_PNTR[] is in sync with PHY value.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */
/*  */
uint32_t adf7030_1__SPI_Xfer_Mem(
    ADI_SPI_HANDLE hSPIDevice,
    MEM_DESCRIPTOR_Type * pMEM,
    bool READ_nWRITE
)
{
    bool exit;
    do{
  
    if(!pMEM->XferMode)
    {
      /* Block Transfers Mode */
             
        uint32_t PHYAddr = pMEM->PhyAddr + pMEM->BlockXfer.Offset;
        
        ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE PNTR_id;
        int32_t PNTR_offset;
        
        /* Get good PNTRid from ADF7030_1_SPI_PNTR[] */
        adf7030_1__SPI_FindMMapPointer( hSPIDevice, PHYAddr, &PNTR_id, &PNTR_offset);
             
        /* 32bits Mode SPI transfers */
        if(!(PHYAddr & 3) && !(pMEM->BlockXfer.Size & 3))
        { 
            /* Setup pointer to Host start of data */
            uint32_t * pHostMem;
            pHostMem = (uint32_t *)pMEM->pBuff + (pMEM->BlockXfer.Offset >> 2);

            /* Valid PHY SPI pointer available, use 32bits bloc pointer access */
            if((PNTR_id) && !(PNTR_offset >> 2))
            {     
                if(READ_nWRITE == true)
                {
                    /* Read from radio PHY */
                    if(adf7030_1__SPI_rd_word_b_p( hSPIDevice, PNTR_id, pMEM->BlockXfer.Size >> 2, pHostMem) != SUCCESS)
                    {
                        return FAILURE;
                    }                    
                }else{
                    /* Write to radio PHY */
                    if(adf7030_1__SPI_wr_word_b_p( hSPIDevice, PNTR_id, pMEM->BlockXfer.Size >> 2, pHostMem) != SUCCESS)
                    {
                        return FAILURE;
                    }  
                }
            }
            /* No valid PHY SPI pointer available, use 32bits bloc address access */
            else
            {
                if(READ_nWRITE == true)
                {
                    /* Read from radio PHY */
                    if(adf7030_1__SPI_rd_word_b_a( hSPIDevice, PHYAddr, pMEM->BlockXfer.Size >> 2, pHostMem) != SUCCESS)
                    {
                        return FAILURE;
                    }                    
                }else{
                    /* Write to radio PHY */
                    if(adf7030_1__SPI_wr_word_b_a( hSPIDevice, PHYAddr, pMEM->BlockXfer.Size >> 2, pHostMem) != SUCCESS)
                    {
                        return FAILURE;
                    }  
                }
            }
        }
        /* 8bits Mode SPI transfers */
        else
        {
            /* Setup pointer to Host start of data */
            uint8_t * pHostMem;
            pHostMem = (uint8_t *)pMEM->pBuff + pMEM->BlockXfer.Offset;

            /* No valid PHY SPI pointer available, use 8bits access with no offset */
            if(PNTR_id == 0)
            {
                /* Change the SPI custom pointer 0 */
                if(adf7030_1__SPI_SetMMapCustomPntr0( hSPIDevice, PHYAddr ) != SUCCESS)
                {
                    return FAILURE;
                }
                
                /* Copy Host MEM memory block to PHY memory block via the SPI */
                if(READ_nWRITE == true)
                {
                    /* Read from radio PHY */
                    if(adf7030_1__SPI_rd_byte_p_a( hSPIDevice, PNTR_CUSTOM0_ADDR, pMEM->BlockXfer.Size, pHostMem) != SUCCESS)
                    {
                        return FAILURE;
                    }                    
                }else{
                    /* Write to radio PHY */
                    if(adf7030_1__SPI_wr_byte_p_a( hSPIDevice, PNTR_CUSTOM0_ADDR, pMEM->BlockXfer.Size, pHostMem) != SUCCESS)
                    {
                        return FAILURE;
                    }  
                }
            }
            /* Valid PHY SPI pointer available, use 8bits access with pointer offset */
            else
            {
                /* Copy Host MEM memory block to PHY memory block via the SPI */
                if(READ_nWRITE == true)
                {
                    /* Read from radio PHY */
                    if(adf7030_1__SPI_rd_byte_b_a( hSPIDevice, PNTR_id, PNTR_offset, pMEM->BlockXfer.Size, pHostMem) != SUCCESS)
                    {
                        return FAILURE;
                    }                    
                }else{
                    /* Write to radio PHY */
                    if(adf7030_1__SPI_wr_byte_b_a( hSPIDevice, PNTR_id, PNTR_offset, pMEM->BlockXfer.Size, pHostMem) != SUCCESS)
                    {
                        return FAILURE;
                    }  
                }
            }              
        }
    }
    else
    {
      /* Random Transfers Mode */
      
    }
    
    
    /* Check if nested MEM transfert is present */
    if(pMEM->pNextBloc)
    {
        /* Move pointer to MEM structure to the following one */
        pMEM = pMEM->pNextBloc;
        exit = false;
    }
    else
    {
        exit = true;
    }
    
  }while(exit == false);
  
  return SUCCESS;
}


/**
 * @brief       Write a single 32bits memory location via the SPI
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  Addr            PHY Address location at which the SPI will write.
 *
 * @param [in]  Value           32bits Value to write.
 *
 * @note                        This function does not support "SPI Error Status"
 *
 * @return      None
 */
void adf7030_1__SPI_SetMem32(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t Value
)
{
    adf7030_1__SPI_SetBytes( hSPIDevice, Addr, Value, 4);
}

/**
 * @brief       Read a single 32bits memory location via the SPI
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  Addr            PHY Address location at which the SPI will write.
 *
 * @note                        This function does not support "SPI Error Status"
 *
 * @return      Value           32bits readback value.
 */
uint32_t adf7030_1__SPI_GetMem32(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr
)
{  
    return(adf7030_1__SPI_GetBytes( hSPIDevice, Addr, 4)); 
}

/**
 * @brief       Write a single 24bits memory location via the SPI
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @note                        This function is changing SPI custom pointer0, 
 *                              ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] and the PHY
 *                              SPI pntr are overwritten. It also does not support
 *                              "SPI Error Status".
 *
 * @param [in]  Addr            PHY Address location at which the SPI will write.
 *
 * @param [in]  Value           24bits Value to write.
 *
 * @return      None
 */
void adf7030_1__SPI_SetMem24(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t Value
)
{
    adf7030_1__SPI_SetBytes( hSPIDevice, Addr, (0xFFFFFF & Value), 3);
}

/**
 * @brief       Read a 24bits memory location via the SPI
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @note                        This function is changing SPI custom pointer0, 
 *                              ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] and the PHY
 *                              SPI pntr are overwritten. It also does not support
 *                              "SPI Error Status".
 *
 * @param [in]  Addr            PHY Address location at which the SPI will write.
 *
 * @return      Value           24bits readback value.
 */
uint32_t adf7030_1__SPI_GetMem24(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr
)
{
    return((0xFFFFFF & adf7030_1__SPI_GetBytes( hSPIDevice, Addr, 3)));
}

/**
 * @brief       Write a single 16bits memory location via the SPI
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @note                        This function is changing SPI custom pointer0, 
 *                              ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] and the PHY
 *                              SPI pntr are overwritten. It also does not support
 *                              "SPI Error Status".
 *
 * @param [in]  Addr            PHY Address location at which the SPI will write.
 *
 * @param [in]  Value           16bits Value to write.
 *
 * @return      None
 */
void adf7030_1__SPI_SetMem16(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint16_t Value
)
{
    adf7030_1__SPI_SetBytes( hSPIDevice, Addr, Value, 2);
}

/**
 * @brief       Read a 16bits memory location via the SPI
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @note                        This function is changing SPI custom pointer0, 
 *                              ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] and the PHY
 *                              SPI pntr are overwritten. It also does not support
 *                              "SPI Error Status".
 *
 * @param [in]  Addr            PHY Address location at which the SPI will write.
 *
 * @return      Value           16bits readback value.
 */
uint16_t adf7030_1__SPI_GetMem16(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr
)
{
    return((uint16_t)adf7030_1__SPI_GetBytes( hSPIDevice, Addr, 2));
}

/**
 * @brief       Write a single 8bits memory location via the SPI
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @note                        This function is changing SPI custom pointer0, 
 *                              ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] and the PHY
 *                              SPI pntr are overwritten. It also does not support
 *                              "SPI Error Status".
 *
 * @param [in]  Addr            PHY Address location at which the SPI will write.
 *
 * @param [in]  Value           8bits Value to write.
 *
 * @return      None
 */
void adf7030_1__SPI_SetMem8(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint8_t Value
)
{
    adf7030_1__SPI_SetBytes( hSPIDevice, Addr, Value, 1);
}

/**
 * @brief       Read a 8bits memory location via the SPI
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @note                        This function is changing SPI custom pointer0, 
 *                              ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] and the PHY
 *                              SPI pntr are overwritten. It also does not support
 *                              "SPI Error Status".
 *
 * @param [in]  Addr            PHY Address location at which the SPI will write.
 *
 * @return      Value           8bits readback value.
 */
uint8_t adf7030_1__SPI_GetMem8(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr
)
{
    return((uint8_t)adf7030_1__SPI_GetBytes( hSPIDevice, Addr, 1));
}

/**
 * @brief       Writes "nbBytes" memory location via the SPI
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @note                        This function is changing SPI custom pointer0, 
 *                              ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] and the PHY
 *                              SPI pntr are overwritten. It also does not support
 *                              "SPI Error Status".
 *
 * @param [in]  Addr            PHY Address location at which the SPI will write.
 *
 * @param [in]  Value           Value to write.
 *
 * @param [in]  nbBytes         Number of Bytes to writes (1, 2, 3 or 4);
 *
 * @return      None
 */
void adf7030_1__SPI_SetBytes(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t Value,
    uint32_t nbBytes
)
{   
    /* Get current SPI custom pntr 0 value */ 
    int32_t AddrDiff = (int32_t)Addr - (int32_t)ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR];
    
    uint8_t Offset;
    
    if((AddrDiff >= 0 ) && (AddrDiff < (255 + nbBytes)))
    {
      /* Here PNTR_CUSTOM0_ADDR is within range, just change the offset */
        Offset = (uint8_t)AddrDiff;
      
    }else{
      /* Here PNTR_CUSTOM0_ADDR is not within range, changing it so */
        
        /* Change Radio SPI pointer 0 address to Addr */
        ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] = ((Addr >> 2) << 2);
    
        /* Update the Radio SPI pointer 0 */
        adf7030_1__SPI_wr_word_b_p( hSPIDevice, PNTR_SETUP_ADDR, 1, &ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR]);      
      
        Offset = Addr & 0x03;
    }
    
  
    if((Offset == 0) && (nbBytes == 4))
    {
      /* Write the 32bit alligned data */ 
        adf7030_1__SPI_wr_word_b_p( hSPIDevice, PNTR_CUSTOM0_ADDR, 1, &Value);
    }else{
      /* Write the nbBytes data */
        adf7030_1__SPI_wr_byte_b_a( hSPIDevice, PNTR_CUSTOM0_ADDR, Offset, nbBytes, (uint8_t *)&Value);
    }
}

/**
 * @brief       Read "nbBytes" memory location via the SPI
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @note                        This function is changing SPI custom pointer0, 
 *                              ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] and the PHY
 *                              SPI pntr are overwritten. It also does not support
 *                              "SPI Error Status".
 *
 * @sa  adf7030_1__SPI_rd_byte_b_a()
 *
 * @param [in]  Addr            PHY Address location at which the SPI will write.
 *
 * @param [in]  nbBytes         Number of Bytes to read (1, 2, 3 or 4);
 *
 * @return      Value           32bits readback value.
 */
uint32_t adf7030_1__SPI_GetBytes(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t nbBytes
)
{
    /* Get current SPI custom pntr 0 value */ 
    int32_t AddrDiff = (int32_t)Addr - (int32_t)ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR];
    
    uint8_t Offset;
    
    if((AddrDiff >= 0 ) && (AddrDiff < (255 + nbBytes)))
    {
      /* Here PNTR_CUSTOM0_ADDR is within range, just change the offset */
        Offset = (uint8_t)AddrDiff;
      
    }else{
      /* Here PNTR_CUSTOM0_ADDR is not within range, changing it so */
        
        /* Change Radio SPI pointer 0 address to Addr */
        ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] = ((Addr >> 2) << 2);
    
        /* Update the Radio SPI pointer 0 */
        adf7030_1__SPI_wr_word_b_p( hSPIDevice, PNTR_SETUP_ADDR, 1, &ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR]);      
      
        Offset = Addr & 0x03;
    }
    
    uint32_t RetVal; 
    if((Offset == 0) && (nbBytes == 4))
    {
      /* Read the 32bit alligned data */  
        adf7030_1__SPI_rd_word_b_p( hSPIDevice, PNTR_CUSTOM0_ADDR, 1, &RetVal);
    }else{
      /* Read the nbBytes data */
        adf7030_1__SPI_rd_byte_b_a( hSPIDevice, PNTR_CUSTOM0_ADDR, Offset, nbBytes, (uint8_t *)&RetVal);
    }
    
    return(RetVal);
}

/**
 * @brief       Generic setting of a Radio PHY bitfield to a value via the SPI
 *
 * @note                        This function does not support "SPI Error Status".
 *                              Call adf7030_1__SPI_GetBytes and adf7030_1__SPI_GetBytes
 *                              and apply appropriate masking to the SPI value readback.
 *
 * @sa  adf7030_1__SPI_GetBytes().
 * @sa  adf7030_1__SPI_SetBytes().
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  Addr            32bit alligned PHY Address location of the bitfield.
 *
 * @param [in]  Pos             Starting bit position withing the 32bits word.
 *
 * @param [in]  Size            Bit lenght of the bitfield.
 *
 * @param [in]  Val             Bitfield value.
 *
 * @return      None            
 */
void adf7030_1__SPI_SetField(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t Pos,
    uint32_t Size,
    uint32_t Val
)
{
    /* Compute Bitfields parameters */
    uint32_t fStartByte = Pos >> 3;
    uint32_t fPos_fromStartByte = Pos - (fStartByte << 3);
    uint32_t fLenBytes = ((Size - 1) >> 3) + 1;
      
    /* Read Bitfield vis SPI */
    uint32_t fVal = adf7030_1__SPI_GetBytes( hSPIDevice, Addr + fStartByte, fLenBytes);
    
    /* Compute the Mask */ 
    uint32_t fMsk = ((1 << Size) - 1) << fPos_fromStartByte;
    
    /* Bound Val to the Mask */
    Val <<= fPos_fromStartByte;
    Val &= fMsk;
    
    /* And SPI returned value with inverted Mask */
    fVal &= (~fMsk);
    
    /* Or SPI returned value with Val*/
    fVal |= Val;
    
    /* Write back Bitfield vis SPI */
    adf7030_1__SPI_SetBytes( hSPIDevice, Addr + fStartByte, fVal, fLenBytes);
}


/**
 * @brief       Generic readback of a bitfield value from the Radio PHY via the SPI
 *
 * @note                        This function does not support "SPI Error Status".
 *                              Call adf7030_1__SPI_GetBytes and apply appropriate
 *                              masking to the SPI value readback.
 *
 * @sa  adf7030_1__SPI_GetBytes().
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  Addr            32bit alligned PHY Address location of the bitfield.
 *
 * @param [in]  Pos             Starting bit position withing the 32bits word.
 *
 * @param [in]  Size            Bit lenght of the bitfield.
 *
 * @return      fVal            Bitfield value
 */
uint32_t adf7030_1__SPI_GetField(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    uint32_t Pos,
    uint32_t Size
)
{
    /* Compute Bitfields parameters */
    uint32_t fStartByte = Pos >> 3;
    uint32_t fPos_fromStartByte = Pos - (fStartByte << 3);
    uint32_t fLenBytes = ((Size - 1) >> 3) + 1;
      
    /* Read Bitfield vis SPI */
    uint32_t fVal = adf7030_1__SPI_GetBytes( hSPIDevice, Addr + fStartByte, fLenBytes);
    
    /* Compute the Mask */ 
    uint32_t fMsk = ((1 << Size) - 1) << fPos_fromStartByte;
    
    /* And SPI returned value with Mask */
    fVal &= fMsk;
    
    /* Shift right by fPos_fromStartByte */
    fVal >>= fPos_fromStartByte;
    
    /* Return Bitfield Value */
    return(fVal);
}

//#define adf7030_1__SPI_GetField(field) 


#endif /* _ADF7030_1__MEM_C_ */

/** @} */ /* End of group adf7030-1__mem PHY Memory Interface */
/** @} */ /* End of group adf7030-1 adf7030-1 Driver */
