/*********************************************************************************

Copyright(c) 2016 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file     adf7030-1__patch.c
*
* @brief    adf7030-1 Patch Memory Access Functions.
*               - SPI Xfer with between Host and Radio.
*               - Inject, Eject
*
* @details  ...
*           ...
*           ...
*/

/** \addtogroup adf7030-1 adf7030-1 Driver
 *  @{
 */

/** \addtogroup adf7030-1__patch PHY Patch Interface
 *  @{
 */
#ifndef _ADF7030_1__PATCH_C_
/*! \cond PRIVATE */
#define _ADF7030_1__PATCH_C_

#include <stddef.h>
#include <string.h>
#include <assert.h>
#include "system.h"
#include "common.h"

#include <drivers/spi/adi_spi.h>
//#include <devices/rf/adf703x/adi_adf7030-1.h>
//#include <devices/rf/adf703x/adi_adf7030-1_reg.h>
#include <devices/rf/adf703x/adf7030-1__spi.h>
#include <devices/rf/adf703x/adf7030-1__mem.h>
#include <devices/rf/adf703x/adf7030-1__patch.h>


#ifdef __ICCARM__
/*
* IAR MISRA C 2004 error suppressions.
*
* Pm140 (rule 11.3): a cast should not be performed between a pointer type and an integral type
*   The rule makes an exception for memory-mapped register accesses.
*
* Pm073 (rule 14.7): a function should have a single point of exit.
* Pm143 (rule 14.7): a function should have a single point of exit at the end of the function.
*   Multiple returns are used for error handling.
*/
#pragma diag_suppress=Pm073,Pm143
#endif /* __ICCARM__ */

/*! \endcond */


/**
 * @brief       Function call to write patch to Radio PHY
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pPatch          Descriptor to the Host patch memory region.
 *
 * @note                        This function only transfert patch data to the PHY. 
 *                              User must enable at a later stage by calling the 
 *                              adf7030_1__InjectPatch() function.
 *                              This function modifies ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR].
 *
 * @return      Status
 *  - #SUCCESS                  If the patch was written successfully in the Host.
 *  - #FAILURE                  [D] If the patch transfert failed.
 */

uint32_t adf7030_1__WritePatch(
    ADI_SPI_HANDLE hSPIDevice,                              
    PATCH_DESCRIPTOR_Type * pPATCH                                
)
{
    return(adf7030_1__XferPatch( hSPIDevice, pPATCH, PHY_MEM_WRITE, false));
}

/**
 * @brief       Function call to readback the patch from Radio PHY
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pPatch          Descriptor to the Host patch memory region.
 *
 * @note                        This function only readback patch data from the PHY. 
 *                              ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] could be modified.
 *
 * @return      Status
 *  - #SUCCESS                  If the patch was readback successfully in the Host.
 *  - #FAILURE                  [D] If the patch transfert failed.
 */

/* Read patch data from Radio PHY */
uint32_t adf7030_1__ReadPatch(
    ADI_SPI_HANDLE hSPIDevice,                              
    PATCH_DESCRIPTOR_Type * pPATCH                                
)
{
    return(adf7030_1__XferPatch( hSPIDevice, pPATCH, PHY_MEM_READ, false));
}

/*PATCH_DESCRIPTOR_Type PATCH_READBACK;

memcpy(&PATCH_READBACK, &PATCH_LCPSM, sizeof(PATCH_DESCRIPTOR_Type));

char buff[PATCH_READBACK.Size];
char *pOffset = buff;

PATCH_READBACK.BLOCKS[0].pData = buff;

for(uint32_t i = 1; i < PATCH_READBACK.nbBlock; i++){
	pOffset += PATCH_READBACK.BLOCKS[i-1].Size;
	PATCH_READBACK.BLOCKS[i].pData = pOffset;
}

adf7030_1__ReadPatch(hSPIDevice, &PATCH_READBACK);
*/


/**
 * @brief       Function call to re-enable the patch if needed
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pPatch          Descriptor to the Host patch memory region.
 *
 * @note                        This function assume the following the patch has
 *                              already been transfered and that its integrity is
 *                              valid. User can check the Ram for the patch presence
 *                              by calling the function adf7030_1__VerifyPatch().
 *
 * @return      Status
 *  - #SUCCESS                  If the Inject sequence was succesfull.
 *  - #FAILURE                  [D] If the Inject failed.
 */

uint32_t adf7030_1__InjectPatch(
    ADI_SPI_HANDLE hSPIDevice,                              
    PATCH_DESCRIPTOR_Type * pPATCH                                
)
{
    /* Write Inject sequence to radio PHY */
    uint32_t InjectSeq = 0x20000000 + pPATCH->InjectSeq;

    if(adf7030_1__SPI_wr_word_b_a(hSPIDevice, 0x20000130, 1, &InjectSeq ) != SUCCESS)
    {
        return FAILURE;
    }
      
    return SUCCESS;
}

/**
 * @brief       Function call to disnable the patch if needed
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pPatch          Descriptor to the Host patch memory region.
 *
 * @note                        This function only disable the patch, it can be 
 *                              re-enable at a later stage by calling the 
 *                              adf7030_1__InjectPatch() function.
 *
 * @return      Status
 *  - #SUCCESS                  If the Eject sequence was succesfull.
 *  - #FAILURE                  [D] If the Eject failed.
 */

uint32_t adf7030_1__EjectPatch(
    ADI_SPI_HANDLE hSPIDevice,                              
    PATCH_DESCRIPTOR_Type * pPATCH                                
)
{
    /* Write Eject sequence to radio PHY */
    uint32_t EjectSeq = 0x20000000 + pPATCH->EjectSeq;
    
    if(adf7030_1__SPI_wr_word_b_a(hSPIDevice, 0x20000130, 1, &EjectSeq ) != SUCCESS)
    {
        return FAILURE;
    }
     
    return SUCCESS;  
}

/**
 * @brief       Function call to verify the integrity of the patch
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pPatch          Descriptor to the Host patch memory region.
 *
 * @note                        This function only disable the patch, it can be 
 *                              re-enable at a later stage by calling the 
 *                              adf7030_1__InjectPatch() function.
 *                              This function modifies ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR].
 *
 * @return      Status
 *  - #SUCCESS                  If the patch in the PHY matches the reference in the Host.
 *  - #FAILURE                  [D] If the patch is different.
 */

uint32_t adf7030_1__VerifyPatch(
    ADI_SPI_HANDLE hSPIDevice,                              
    PATCH_DESCRIPTOR_Type * pPATCH                                
)
{
  return(adf7030_1__XferPatch( hSPIDevice, pPATCH, PHY_MEM_READ, true));
}


/**
 * @brief       Function to write Patch Information to the PHY Radio
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pPathInfo       Pointer to Host patch descriptor memory region.
 *
 * @param [in]  PhyAddr         Address location of patch descriptor in the 
 *                              PHY Radio memory space.
 *
 * @note                        This function modifies ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR].
 *
 * @return      Status
 *  - #SUCCESS                  If the Patch Information has been read back into pPathInfo.
 *  - #FAILURE                  [D] If the Patch Information could not be read.
 */

uint32_t adf7030_1_WritePatchInfo(
    ADI_SPI_HANDLE hSPIDevice,                              
    version_info_t * pPathInfo,
    uint32_t PhyAddr
)
{
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE PNTR_id;
    int32_t PNTR_offset;
    
    /* Get good PNTRid from ADF7030_1_SPI_PNTR[] */
    adf7030_1__SPI_FindMMapPointer( hSPIDevice, PhyAddr, &PNTR_id, &PNTR_offset);
    
    /* No valid PHY SPI pointer available, use 8bits access with no offset */
    if(PNTR_id == 0)
    {
       /* Change the SPI custom pointer 0 */
       if(adf7030_1__SPI_SetMMapCustomPntr0( hSPIDevice, PhyAddr ) != SUCCESS)
       {
           return FAILURE;
       }
       
       /* Read from radio PHY */
       if(adf7030_1__SPI_wr_byte_p_a( hSPIDevice, PNTR_CUSTOM0_ADDR, sizeof(version_info_t), (uint8_t *)pPathInfo) != SUCCESS)
       {
           return FAILURE;
       }                    
    }
    /* Valid PHY SPI pointer available, use 8bits access with pointer offset */
    else
    {
        /* Read from radio PHY */
        if(adf7030_1__SPI_wr_byte_b_a( hSPIDevice, PNTR_id, PNTR_offset, sizeof(version_info_t), (uint8_t *)pPathInfo) != SUCCESS)
        {
            return FAILURE;
        }                    
    }
    
    return SUCCESS;
}


/**
 * @brief       Function to read Patch Information from the PHY Radio
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in|out] pPathInfo       Pointer to Host patch descriptor memory region.
 *
 * @param [in]  PhyAddr         Address location of patch descriptor in the 
 *                              PHY Radio memory space.
 *
 * @note                        This function modifies ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR].
 *
 * @return      Status
 *  - #SUCCESS                  If the Patch Information has been read back into pPathInfo.
 *  - #FAILURE                  [D] If the Patch Information could not be read.
 */

uint32_t adf7030_1_ReadPatchInfo(
    ADI_SPI_HANDLE hSPIDevice,                              
    version_info_t * pPathInfo,
    uint32_t PhyAddr
)
{
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE PNTR_id;
    int32_t PNTR_offset;
    
    /* Get good PNTRid from ADF7030_1_SPI_PNTR[] */
    adf7030_1__SPI_FindMMapPointer( hSPIDevice, PhyAddr, &PNTR_id, &PNTR_offset);
    
    /* No valid PHY SPI pointer available, use 8bits access with no offset */
    if(PNTR_id == 0)
    {
       /* Change the SPI custom pointer 0 */
       if(adf7030_1__SPI_SetMMapCustomPntr0( hSPIDevice, PhyAddr ) != SUCCESS)
       {
           return FAILURE;
       }
       
       /* Read from radio PHY */
       if(adf7030_1__SPI_rd_byte_p_a( hSPIDevice, PNTR_CUSTOM0_ADDR, sizeof(version_info_t), (uint8_t *)pPathInfo) != SUCCESS)
       {
           return FAILURE;
       }                    
    }
    /* Valid PHY SPI pointer available, use 8bits access with pointer offset */
    else
    {
        /* Read from radio PHY */
        if(adf7030_1__SPI_rd_byte_b_a( hSPIDevice, PNTR_id, PNTR_offset, sizeof(version_info_t), (uint8_t *)pPathInfo) != SUCCESS)
        {
            return FAILURE;
        }                    
    }
    
    return SUCCESS;
}

/**
 * @brief       Generic Patch transfers function between Host and the Radio PHY
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pPatch          Descriptor to the Host patch memory region.
 *
 * @param [in]  READ_nWRITE     Boolean  -TRUE to write Patch from Host to the Radio PHY
 *                                       -FALSE to read Patch from Radio PHY back to Host
 *
 * @param [in]  CHECK_ONLY      Boolean  -TRUE to check PHY patch versus Host Reference
 *                                       -FALSE [D] to perform READ_nWRITE operation
 *
 * @note                        This function can modify ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR].
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__XferPatch(
    ADI_SPI_HANDLE hSPIDevice,
    PATCH_DESCRIPTOR_Type * pPATCH,
    bool READ_nWRITE,
    bool CHECK_ONLY
)
{   
    for(uint32_t i = 0; i < pPATCH->nbBlock; i++)
    {
        /* Block Transfers Mode */
             
        uint32_t PHYAddr = pPATCH->BLOCKS[i].Addr;
        
        /* Crude check to skip transfert of Patch Informations */
        if(PHYAddr == pPATCH->InfoAddr)
        {
          break;
        }
        
        ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE PNTR_id;
        int32_t PNTR_offset;
        
        /* Get good PNTRid from ADF7030_1_SPI_PNTR[] */
        adf7030_1__SPI_FindMMapPointer( hSPIDevice, PHYAddr, &PNTR_id, &PNTR_offset);
             
        /* Lets just use 8bits Mode SPI transfers for patch*/
        {
            /* Setup pointer to Host start of data */
            uint8_t * pHostMem;
            pHostMem = (uint8_t *)pPATCH->BLOCKS[i].pData;
            
            uint32_t size = pPATCH->BLOCKS[i].Size;

            /* No valid PHY SPI pointer available, use 8bits access with no offset */
            if(PNTR_id == 0)
            {
                /* Change the SPI custom pointer 0 */
                if(adf7030_1__SPI_SetMMapCustomPntr0( hSPIDevice, PHYAddr ) != SUCCESS)
                {
                    return FAILURE;
                }
                
                if(CHECK_ONLY)
                {
                  /* Read from radio PHY */
                    if(adf7030_1__SPI_rd_cmp_byte_p_a( hSPIDevice, PNTR_CUSTOM0_ADDR, size, NULL, pHostMem) != SUCCESS)
                    {
                         return FAILURE;
                    } 
                }
                else
                {
                    /* Copy Host MEM memory block to PHY memory block via the SPI */
                    if(READ_nWRITE == true)
                    {
                        /* Read from radio PHY */
                        if(adf7030_1__SPI_rd_byte_p_a( hSPIDevice, PNTR_CUSTOM0_ADDR, size, pHostMem) != SUCCESS)
                        {
                            return FAILURE;
                        }                    
                    }else{
                        /* Write to radio PHY */
                        if(adf7030_1__SPI_wr_byte_p_a( hSPIDevice, PNTR_CUSTOM0_ADDR, size, pHostMem) != SUCCESS)
                        {
                            return FAILURE;
                        }  
                    }
                }
            }
            /* Valid PHY SPI pointer available, use 8bits access with pointer offset */
            else
            {
                if(CHECK_ONLY)
                {
                  /* Read from radio PHY */
                    if(adf7030_1__SPI_rd_cmp_byte_b_a( hSPIDevice, PNTR_id, PNTR_offset, size, NULL, pHostMem) != SUCCESS)
                    {
                         return FAILURE;
                    } 
                }
                else
                {
                  /* Copy Host MEM memory block to PHY memory block via the SPI */
                    if(READ_nWRITE == true)
                    {
                        /* Read from radio PHY */
                        if(adf7030_1__SPI_rd_byte_b_a( hSPIDevice, PNTR_id, PNTR_offset, size, pHostMem) != SUCCESS)
                        {
                            return FAILURE;
                        }                    
                    }else{
                        /* Write to radio PHY */
                        if(adf7030_1__SPI_wr_byte_b_a( hSPIDevice, PNTR_id, PNTR_offset, size, pHostMem) != SUCCESS)
                        {
                            return FAILURE;
                        }  
                    }
                }
            }
        }
    }  
    
    return SUCCESS;
}


#endif /* _ADF7030_1__PATCH_C_ */

/** @} */ /* End of group adf7030-1__patch PHY Patch Interface */
/** @} */ /* End of group adf7030-1 adf7030-1 Driver */
