/*********************************************************************************

Copyright(c) 2016 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file     adf7030-1_spi.c
*
* @brief    Spi communication with between Glue and Radio.
*
* @details  ...
*           ...
*           ...
*/

/** \addtogroup adf7030-1 adf7030-1 Driver
 *  @{
 */

/** \addtogroup adf7030-1__spi SPI Interface layer
 *  @{
 */

#ifndef _ADF7030_1__SPI_C_

/*! \cond PRIVATE */
#define _ADF7030_1__SPI_C_

#include <stddef.h>
#include <string.h>
#include <assert.h>
#include "system.h"
#include "common.h"

#include <drivers/spi/adi_spi.h>
//#include <devices/rf/adf703x/adi_adf7030-1.h>
//#include <devices/rf/adf703x/adi_adf7030-1_reg.h>
#include <devices/rf/adf703x/adf7030-1__spi.h>

#ifdef __ICCARM__
/*
* IAR MISRA C 2004 error suppressions.
*
* Pm140 (rule 11.3): a cast should not be performed between a pointer type and an integral type
*   The rule makes an exception for memory-mapped register accesses.
*
* Pm073 (rule 14.7): a function should have a single point of exit.
* Pm143 (rule 14.7): a function should have a single point of exit at the end of the function.
*   Multiple returns are used for error handling.
*/
#pragma diag_suppress=Pm073,Pm143
#endif /* __ICCARM__ */

/*! \endcond */

/* Memory required for SPI driver */
uint8_t SPIMem[ADI_SPI_MEMORY_SIZE];

/* SPI device handle */
ADI_SPI_HANDLE              hSPIDevice;

/* SPI transfers buffers */
 uint8_t      spi_txbuf[ADF7030_1_SPI_BUFFER_SIZE];
 uint8_t      spi_rxbuf[ADF7030_1_SPI_BUFFER_SIZE];


/**
 * @brief       Initializes SPI device for communication with the adf7030-1 PHY
 *
 * @note        This function open the SPI peripheral in "blocking mode" 
 *              without DMA transfert assistance. blablabla... 
 *              
 * @return      Status
 *  - #SUCCESS                  If SPI peripheral was succesfully initialise
 *  - #FAILURE                  [D] SPI peripheral failed to initialise
 *
 * @sa  adf7030_1__SPI_UnInit().
 */

uint32_t adf7030_1__SPI_Init(
    void
)
{
    if(adi_spi_Open(SPI_DEV_NUM,
                    SPIMem,
                    ADI_SPI_MEMORY_SIZE,
                    &hSPIDevice) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }

    /* Enable underflow errors */
    if(adi_spi_SetTransmitUnderflow(hSPIDevice, true) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Set the SPI clock rate */
    if(adi_spi_SetBitrate(hSPIDevice, SPI_DEV_BITRATE) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Set the chip select */
    if(adi_spi_SetChipSelect(hSPIDevice, ADI_SPI_CS3) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Set the SPI clock phase */
    if(adi_spi_SetClockPhase(hSPIDevice, false) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }    
    
    /* Set the continuous transfer mode */
    if(adi_spi_SetContinousMode(hSPIDevice, true) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Disable DMA transfer mode */
    if(adi_spi_EnableDmaMode(hSPIDevice, false) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }

    return SUCCESS;
}

/**
 * @brief       Uninitializes SPI device used to communication with the adf7030-1 PHY
 *
 * @note        This function release and deallocate memory used by the SPI peripheral
 *              
 * @return      Status
 *  - #SUCCESS                  If SPI peripheral was succesfully released
 *  - #FAILURE                  [D] SPI peripheral failed to release
 *
 * @sa  adf7030_1__SPI_Init().
 */

uint32_t adf7030_1__SPI_UnInit(
    void
)
{
    if(adi_spi_Close(hSPIDevice) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }
    
    return SUCCESS;
}

/**
 * @brief       Change SPI Host to PHY Communication bitrate
 *
 * @note        Default speed is set to 3.5MBps, anything faster should really 
 *              used DMA operation mode which is corrently not supported
 * 
 * @param [in]  RateHz          Target bitrate (in Hz) for SPI communication.
 *
 * @return      Status
 *  - #SUCCESS                  If SPI peripheral was succesfully released
 *  - #FAILURE                  [D] SPI peripheral failed to release
 *
 * @sa  adi_spi_SetBitrate().
 */

uint32_t adf7030_1__SPI_SetSpeed(
    const uint32_t RateHz
)
{
    if(adi_spi_SetBitrate(hSPIDevice,RateHz) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }
    
    return SUCCESS;
}

/**
 * @brief       Set SPI Radio mmap pointers access
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pPNTR           Pointer to location of mmap user defined pointer 0
 *                              eg. ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR].
 *
 * @return      Status
 *  - #SUCCESS                  If the configuration of mmap pointers was succesfull.
 *  - #FAILURE                  [D] If the configuration of mmap pointers failed.
 */

uint32_t adf7030_1__SPI_SetMMapPointers(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t * pPNTR
)
{
    return(adf7030_1__SPI_wr_word_b_p( hSPIDevice, PNTR_SETUP_ADDR, 3, pPNTR));
}

/**
 * @brief       Get SPI Radio mmap pointers access
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [out]  pPNTR          Pointer to location of mmap user defined pointer 0
 *                              eg. ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR].
 *
 * @return      Status
 *  - #SUCCESS                  If the readback of mmap pointers was succesfull.
 *  - #FAILURE                  [D] If the readback failed.
 */

uint32_t adf7030_1__SPI_GetMMapPointers(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t * pPNTR
)
{
    return(adf7030_1__SPI_rd_word_b_p( hSPIDevice, PNTR_SETUP_ADDR, 3, pPNTR));
}

/**
 * @brief       This function return the best Pointer id + offset for subsequent
 *              SPI transfert to the PHY "Addr" provided.
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  Addr            Desired Radio PHY address.
 *
 * @param [out] pPNTR_ID        Pointer to PNTR_ID variable to access "Addr" location of mmap
 *                              eg. ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR].
 *
 * @param [out] ByteOffset      Pointer to the positive byte offset variable between
 *                              "Addr" and ADF7030_1_SPI_PNTR[pPNTR_ID].
 *
 * @return      None
 */

void adf7030_1__SPI_FindMMapPointer(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE * pPNTR_ID,
    int32_t * ByteOffset
)
{
    uint32_t mId = 0;
    int32_t diff;
    
    *pPNTR_ID = 0;
    
    /* Search through ADF7030_1_SPI_PNTR[] for a Pointer within range of "Addr" */
    for(uint32_t id = PNTR_SRAM_ADDR; id <= PNTR_CUSTOM2_ADDR; id++)
    {
        /* Calculate byte offset with the current ADF7030_1_SPI_PNTR[id] */
        diff = (int32_t)Addr - (int32_t)ADF7030_1_SPI_PNTR[id];
        
        /* Stop search if offset within range */
        if((diff > 0) && (diff <= 255))
        {
           mId = id;
           *ByteOffset = diff;
        }
        else if(diff == 0)
        {
           mId = id;
           *ByteOffset = diff;
        }
    }
    
    if(mId)
    {
        /* ADF7030_1_SPI_PNTR[] contains "Addr", Write Back pPNTR_ID variable */
        *pPNTR_ID = mId;
    }   
}  


/**
 * @brief       Change the SPI Radio mmap custom pointers 0 location
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  Addr            Value of the ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR].
 *
 * @return      Status
 *  - #SUCCESS                  If the "Addr" is within range for the SPI transfer.
 *  - #FAILURE                  [D] If no PointerID is available.
 */

uint32_t adf7030_1__SPI_SetMMapCustomPntr0(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t Addr
)
{
    /* Update the host ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] to Addr value */
    ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] = Addr;
    
    /* Update the Radio PHY spi pointer 0 */
    return(adf7030_1__SPI_wr_word_b_p( hSPIDevice, PNTR_SETUP_ADDR, 1, &ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR]));
}


/**
 * @brief       Write a number of word(s) from Host to memory of the adf7030-1
 *              Generic 32bits bloc address write to memory (HRM 1.4.4.2.3).
 *              This Function should be used when writing a large block of 
 *              32bits word to the radio (above ADI_SPI_MEMORY_SIZE).
 *
 * @note        If the number of words to be transfered exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the writting operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  AddrIn          Address location at which the SPI write should 
 *                              start.
 *
 * @param [in]  num_xfrs        Number of 32bits word write to be performed.
 *
 * @param [in]  pDataIn         Pointer to the data block to be transmitted.
 *                              On exit, pDataIn is pointing at the location of 
 *                              the AddrIn + num_xfrs of the host memory. 
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 *
 * @sa  blabla()
 */

uint32_t adf7030_1__SPI_wr_word_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint32_t *pDataIn
)
{
    /* Transmit SPI command - use 3bytes offset to keep rest of data alligned on word boundary */
    spi_txbuf[3] =  ADF703x_SPI_MEM_WRITE | 
                    ADF703x_SPI_MEM_BLOCK |
                    ADF703x_SPI_MEM_ADDRESS |
                    ADF703x_SPI_MEM_LONG;
					
    /* Write the starting address in the spi_txbuf */
    *(((uint32_t *)spi_txbuf) + 1) = __REV(AddrIn);

    return(adf7030_1__SPI_Block_Xfer( hSPIDevice, 3, 0, AddrIn, num_xfrs << 2, pDataIn, NULL));   
 }

/**
 * @brief       Read a number of word(s) from the adf7030-1 to memory of Host
 *              Generic 32bits bloc address read from memory (HRM 1.4.4.2.3).
 *              This Function should be used when reading a large block of 
 *              32bits word from the radio (above ADI_SPI_MEMORY_SIZE).
 *
 * @note        If the number of words to read exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the reading operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  AddrIn          Address location at which the SPI write should 
 *                              start.
 *
 * @param [in]  num_xfrs        Number of 32bits word write to be performed.
 *
 * @param [out] pDataOut        Pointer to the write back location in the host.
 *                              On exit, pDataOut is pointing at the location of 
 *                              the AddrIn + num_xfrs of the host memory. 
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 *
 * @sa  blabla()
 */

uint32_t adf7030_1__SPI_rd_word_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint32_t * pDataOut
)
{   
    return(adf7030_1__SPI_rd_cmp_word_b_a(hSPIDevice, AddrIn, num_xfrs, pDataOut, NULL));
}

/**
 * @brief       Read & Compare a number of word(s) from the adf7030-1 to memory of Host
 *              Generic 32bits bloc address read from memory (HRM 1.4.4.2.3).
 *              This Function should be used when reading a large block of 
 *              32bits word from the radio (above ADI_SPI_MEMORY_SIZE).
 *
 * @note        If the number of words to read exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the reading operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  AddrIn          Address location at which the SPI write should 
 *                              start.
 *
 * @param [in]  num_xfrs        Number of 32bits word write to be performed.
 *
 * @param [out] pDataOut        Pointer to the write back location in the host.
 *                              On exit, pDataOut is pointing at the location of 
 *                              the AddrIn + num_xfrs of the host memory. 
 *
 * @param [in]  pDataRef        Pointer to the start of the Ref Host data block.
 *                              If set, the SPI readback data will be compared with the 
 *                              data pointed by pDataRef. This can be used to compare
 *                              data between Host and PHY directly within the readback.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 *
 * @sa  blabla()
 */

uint32_t adf7030_1__SPI_rd_cmp_word_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint32_t * pDataOut,
    uint32_t * pDataRef
)
{   
    /* Transmit SPI command - use 1bytes offset to keep rest of data alligned on word boundary */
    spi_txbuf[1] =  ADF703x_SPI_MEM_READ | 
                    ADF703x_SPI_MEM_BLOCK |
                    ADF703x_SPI_MEM_ADDRESS |
                    ADF703x_SPI_MEM_LONG;

    /* Write the starting address in the spi_txbuf */
    *((uint16_t *)spi_txbuf + 1) = __REV16(*((uint16_t *)&AddrIn + 1));
    *((uint16_t *)spi_txbuf + 2) = __REV16(*((uint16_t *)&AddrIn + 0));
    *((uint16_t *)spi_txbuf + 3) = 0xFFFF;

    return(adf7030_1__SPI_Block_Xfer( hSPIDevice, 1, 0, AddrIn, num_xfrs << 2, pDataOut, pDataRef));   
}

/**
 * @brief       Write a number of word(s) from Host to memory of the adf7030-1
 *              starting at location pointed by "pntr"
 *              Generic 32bits bloc pointer write to memory (HRM 1.4.4.2.5/9)
 *              This Function should be used when writing a large block of 
 *              32bits word to the radio (above ADI_SPI_MEMORY_SIZE).
 *
 * @note        If the number of words to be transfered exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the writting operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pntrID          SPI pointer ID to use as base address for writing 
 *                              the block of data.
 *
 * @param [in]  num_xfrs        Number of 32bits word write to be performed.
 *
 * @param [in]  pDataIn         Pointer to the data block to be transmitted.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 *
 * @sa  blabla()
 */

uint32_t adf7030_1__SPI_wr_word_b_p(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint32_t * pDataIn
)
{       
    /* Transmit SPI command - use 3bytes offset to keep rest of data alligned on word boundary */
    spi_txbuf[3] =  ADF703x_SPI_MEM_WRITE | 
                    ADF703x_SPI_MEM_BLOCK |
                    ADF703x_SPI_MEM_POINTER |
                    ADF703x_SPI_MEM_LONG |
                    pntrID;

    return(adf7030_1__SPI_Block_Xfer( hSPIDevice, 3, pntrID, 0, num_xfrs << 2, pDataIn, NULL));
}

/**
 * @brief       Read a number of word(s) from memory of the adf7030-1 to Host
 *              starting at location pointed by "pntr"
 *              Generic 32bits bloc pointer read from memory (HRM 1.4.4.2.5/9)
 *
 * @note        If the number of words to be received exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the writting operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pntrID          SPI pointer ID to use as base address for reading 
 *                              the block of data.
 *
 * @param [in]  num_xfrs        Number of 32bits word read to be performed.
 *
 * @param [out]  pDataOut       Pointer to the write back location in the host.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 *
 * @sa  blabla()
 */

uint32_t adf7030_1__SPI_rd_word_b_p(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint32_t * pDataOut
)
{
    return(adf7030_1__SPI_rd_cmp_word_b_p(hSPIDevice, pntrID, num_xfrs, pDataOut, NULL));
}

/**
 * @brief       Read & Compare a number of word(s) from memory of the adf7030-1 to Host
 *              starting at location pointed by "pntr"
 *              Generic 32bits bloc pointer read from memory (HRM 1.4.4.2.5/9)
 *
 * @note        If the number of words to be received exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the writting operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pntrID          SPI pointer ID to use as base address for reading 
 *                              the block of data.
 *
 * @param [in]  num_xfrs        Number of 32bits word read to be performed.
 *
 * @param [out]  pDataOut       Pointer to the write back location in the host.
 *
 * @param [in]  pDataRef        Pointer to the start of the Ref Host data block.
 *                              If set, the SPI readback data will be compared with the 
 *                              data pointed by pDataRef. This can be used to compare
 *                              data between Host and PHY directly within the readback.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 *
 * @sa  blabla()
 */

uint32_t adf7030_1__SPI_rd_cmp_word_b_p(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint32_t * pDataOut,
    uint32_t * pDataRef
)
{
    /* Transmit SPI command - use 1bytes offset to keep rest of data alligned on word boundary */
    spi_txbuf[1] =  ADF703x_SPI_MEM_READ | 
                    ADF703x_SPI_MEM_BLOCK |
                    ADF703x_SPI_MEM_POINTER |
                    ADF703x_SPI_MEM_LONG |
                    pntrID;
					
	/* Add 16bits of NOPs in spi_txbuf */
    *((uint16_t *)spi_txbuf + 1) = 0xFFFF;
    
    return(adf7030_1__SPI_Block_Xfer( hSPIDevice, 1, pntrID, 0, num_xfrs << 2, pDataOut, pDataRef));
}


/**
 * @brief       Write a number of byte(s) from Host to memory of the adf7030-1
 *              starting at location pointed by "pntr" + AddrIn
 *              Generic 8bits bloc address write to memory (HRM 1.4.4.2.4)
 *
 * @note        If the number of bytes to be transmitted exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the writting operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pntrID          SPI pointer ID to use as base address for writing 
 *                              the block of data.
 *
 * @param [in]  AddrIn          Address location at which the SPI write should 
 *                              start.
 *
 * @param [in]  num_xfrs        Number of byte write to be performed.
 *
 * @param [in]  pDataIn         Pointer to the data block to be transmitted.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__SPI_wr_byte_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint8_t * pDataIn
)
{    
    /* Transmit SPI command - use 2bytes offset to keep rest of data alligned on word boundary */
    spi_txbuf[2] =  ADF703x_SPI_MEM_WRITE | 
                    ADF703x_SPI_MEM_BLOCK |
                    ADF703x_SPI_MEM_ADDRESS |
                    ADF703x_SPI_MEM_SHORT |
                    pntrID;
    
    /* Set Block offset from pntrID */
    spi_txbuf[3] = (uint8_t)AddrIn;

    return(adf7030_1__SPI_Block_Xfer( hSPIDevice, 2, pntrID, AddrIn, num_xfrs, pDataIn, NULL));
}

/**
 * @brief       Read a number of bytes(s) from memory of the adf7030-1 to Host
 *              starting at location pointed by "pntr" + AddrIn offset
 *              Generic 8bits bloc address read from memory (HRM 1.4.4.2.4)
 *
 * @note        If the number of bytes to be received exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the writting operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pntrID          SPI pointer ID to use as base address for reading 
 *                              the block of data.
 *
 * @param [in]  AddrIn          Address location at which the SPI read should 
 *                              start.
 *
 * @param [in]  num_xfrs        Number of bytes read to be performed.
 *
 * @param [out]  pDataOut       Pointer to the data block to be transmitted.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__SPI_rd_byte_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint8_t * pDataOut
)
{    
    return(adf7030_1__SPI_rd_cmp_byte_b_a(hSPIDevice, pntrID, AddrIn, num_xfrs, pDataOut, NULL));
}


/**
 * @brief       Read & Compare a number of bytes(s) from memory of the adf7030-1 to Host
 *              starting at location pointed by "pntr" + AddrIn offset
 *              Generic 8bits bloc address read from memory (HRM 1.4.4.2.4)
 *
 * @note        If the number of bytes to be received exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the writting operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pntrID          SPI pointer ID to use as base address for reading 
 *                              the block of data.
 *
 * @param [in]  AddrIn          Address location at which the SPI read should 
 *                              start.
 *
 * @param [in]  num_xfrs        Number of bytes read to be performed.
 *
 * @param [out]  pDataOut       Pointer to the data block to be transmitted.
 *
 * @param [in]  pDataRef        Pointer to the start of the Ref Host data block.
 *                              If set, the SPI readback data will be compared with the 
 *                              data pointed by pDataRef. This can be used to compare
 *                              data between Host and PHY directly within the readback.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__SPI_rd_cmp_byte_b_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t AddrIn,
    uint32_t num_xfrs,
    uint8_t * pDataOut,
    uint8_t * pDataRef
)
{    
    /* Transmit SPI command - use 1bytes offset to keep rest of data alligned on word boundary */
    spi_txbuf[1] =  ADF703x_SPI_MEM_READ | 
                    ADF703x_SPI_MEM_BLOCK |
                    ADF703x_SPI_MEM_ADDRESS |
                    ADF703x_SPI_MEM_SHORT |
                    pntrID;
    
    /* Set Block offset from pntrID */
    spi_txbuf[2] = (uint8_t)AddrIn;
    
    /* Add 8bits of NOPs in spi_txbuf */
    spi_txbuf[3] = 0xFF;
    
    return(adf7030_1__SPI_Block_Xfer( hSPIDevice, 1, pntrID, AddrIn, num_xfrs, pDataOut, pDataRef));
}

/**
 * @brief       Write a number of byte(s) from Host to memory of the adf7030-1
 *              starting at location pointed by "pntr".
 *              Generic 8bits bloc pointer write to memory (HRM 1.4.4.2.6/10)
 *              This Function should be used when writing a large block of 
 *              8bits word to the radio (above ADI_SPI_MEMORY_SIZE).
 *
 * @note        If the number of bytes to be transmitted exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the writting operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pntrID          SPI pointer ID to use as base address for reading 
 *                              the block of data.
 *
 * @param [in]  num_xfrs        Number of byte write to be performed.
 *
 * @param [in]  pDataIn         Pointer to the data block to be transmitted.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__SPI_wr_byte_p_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint8_t * pDataIn
)
{  
    /* Transmit SPI command - use 3bytes offset to keep rest of data alligned on word boundary */
    spi_txbuf[3] =  ADF703x_SPI_MEM_WRITE | 
                    ADF703x_SPI_MEM_BLOCK |
                    ADF703x_SPI_MEM_POINTER |
                    ADF703x_SPI_MEM_SHORT |
                    pntrID;

    return(adf7030_1__SPI_Block_Xfer( hSPIDevice, 3, pntrID, 0, num_xfrs, pDataIn, NULL));
}

/**
 * @brief       Read a number of bytes(s) from memory of the adf7030-1 to Host
 *              starting at location pointed by "pntr"
 *              Generic 8bits bloc pointer read from memory (HRM 1.4.4.2.6/10)
 *              This Function should be used when writing a large block of 
 *              unaligned data to the radio (above ADI_SPI_MEMORY_SIZE).
 *
 * @note        If the number of bytes to be received exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the writting operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pntrID          SPI pointer ID to use as base address for reading 
 *                              the block of data.
 *
 * @param [in]  num_xfrs        Number of byte write to be performed.
 *
 * @param [out]  pDataOut       Pointer to the data block to be transmitted.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__SPI_rd_byte_p_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint8_t * pDataOut
)
{   
    return(adf7030_1__SPI_rd_cmp_byte_p_a(hSPIDevice, pntrID, num_xfrs, pDataOut, NULL));
}


/**
 * @brief       Read & Compare a number of bytes(s) from memory of the adf7030-1 to Host
 *              starting at location pointed by "pntr"
 *              Generic 8bits bloc pointer read from memory (HRM 1.4.4.2.6/10)
 *              This Function should be used when writing a large block of 
 *              unaligned data to the radio (above ADI_SPI_MEMORY_SIZE).
 *
 * @note        If the number of bytes to be received exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the writting operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  pntrID          SPI pointer ID to use as base address for reading 
 *                              the block of data.
 *
 * @param [in]  num_xfrs        Number of byte write to be performed.
 *
 * @param [out]  pDataOut       Pointer to the data block to be transmitted.
 *
 * @param [in]  pDataRef        Pointer to the start of the Ref Host data block.
 *                              If set, the SPI readback data will be compared with the 
 *                              data pointed by pDataRef. This can be used to compare
 *                              data between Host and PHY directly within the readback.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__SPI_rd_cmp_byte_p_a(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE pntrID,
    uint32_t num_xfrs,
    uint8_t * pDataOut,
    uint8_t * pDataRef
)
{   
    /* Transmit SPI command - use 2bytes offset to keep rest of data alligned on word boundary */
    spi_txbuf[2] =  ADF703x_SPI_MEM_READ | 
                    ADF703x_SPI_MEM_BLOCK |
                    ADF703x_SPI_MEM_POINTER |
                    ADF703x_SPI_MEM_SHORT |
                    pntrID;
    
    /* Add 8bits of NOPs in spi_txbuf */
    spi_txbuf[3] = 0xFF;

    return(adf7030_1__SPI_Block_Xfer( hSPIDevice, 2, pntrID, 0, num_xfrs, pDataOut, pDataRef));        
}
//Random address access (Not implemented yet)
/**
 * @brief       Write a number of word(s) from Host to memory of the adf7030-1
 *              Generic 32bits random address write to memory (HRM 1.4.4.2.7).
 *              This Function should be used when writing a large block of 
 *              32bits word to the radio (above ADI_SPI_MEMORY_SIZE).
 *
 * @note        If the number of words to be transfered exceeds the maximum SPI buffer 
 *              size allocated, the spi transfert is automatically cut
 *              into smaller sub spi transaction until the writting operation 
 *              is completed. No buffer owerflow checking.
 *              
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  AddrIn          Address location (of the Radio) at which the 
 *                              SPI write should start writing words.
 *
 * @param [in]  pDataInOffset   Pointer to the word alligned offset buffer. 
 *
 * @param [in]  pDataIn         Pointer to the memory block from which data
 *                              will be transfert according to the offset value.
 *                              phy_data[AddrIn + AddrOffset] = pDataIn[AddrOffset].
 *                              On exit, pDataIn is pointing at the address of 
 *                              the last transfered word on the host memory. 
 *
 * @param [in]  num_xfrs        Number of 32bits word write to be performed.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 *
 * @sa  blabla()
 */

uint32_t adf7030_1__SPI_wr_word_r_a(
    ADI_SPI_HANDLE hSPIDevice,
    uint32_t AddrIn,
    uint32_t *pHostOffset,
    uint32_t *pAddrIn,
    uint32_t *pDataIn,
    uint32_t num_xfrs
)
{
    /* Crude checking of maximum transfert size */
    if( num_xfrs > (ADF7030_1_SPI_MAX_TRX_SIZE >> 3))
    {
        return FAILURE;
    }
    
    /* Are you using preloaded target address location? */
    if(pHostOffset == NULL)
    {
      
    }

    /* Transmit SPI command - use 3bytes offset to keep rest of data alligned on word boundary */
    spi_txbuf[3] =  ADF703x_SPI_MEM_WRITE | 
                    ADF703x_SPI_MEM_RANDOM |
                    ADF703x_SPI_MEM_ADDRESS |
                    ADF703x_SPI_MEM_LONG;
 
    /* Initiate the SPI transfer */
    if(adf7030_1__SPI_word_random_transaction( &hSPIDevice,       //SPI handle
                                              3,                  //SPI trx buffer offset
                                              num_xfrs,           //SPI nb_word to transfert
                                              pAddrIn,		  //pointer to address buffer
                                              pDataIn,            //pointer to data buffer
                                              true
                                            ) != SUCCESS)
    {
        return FAILURE;
    }
            
    return SUCCESS;
}

//(optional) static uint32_t adf7030_1__SPI_wr_word_r_a(ADI_SPI_HANDLE hSPIDevice, uint32_t * pAddrIn, uint32_t * pDataIn, uint32_t num_xfrs);			// (HRM 1.4.4.2.7)
//(optional) static uint32_t adf7030_1__SPI_rd_word_r_a(ADI_SPI_HANDLE hSPIDevice, uint32_t * pAddrIn, uint32_t * pDataOut, uint32_t num_xfrs);			// (HRM 1.4.4.2.7)
//(optional) static uint32_t adf7030_1__SPI_wr_byte_r_a(ADI_SPI_HANDLE hSPIDevice, uint32_t pID, uint8_t * pOffset, uint8_t * pDataIn, uint32_t num_xfrs);	// (HRM 1.4.4.2.8)
//(optional) static uint32_t adf7030_1__SPI_rd_byte_r_a(ADI_SPI_HANDLE hSPIDevice, uint32_t pID, uint8_t * pOffset, uint8_t * pDataOut, uint32_t num_xfrs);	// (HRM 1.4.4.2.8)

/**
 * @brief       Generic function to read or write a number of bytes between
 *              the Host and the PHY adf703x.
 *
 * @note        If the number of bytes to be transfered exceeds the maximum SPI  
 *              buffer size allocated, the spi transfert is automatically cut
 *              down into smaller sub spi transaction until the transfer operation 
 *              is completed. No buffer owerflow or boundary check is done.
 *              
 *              
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  cmdOffset       Location within the spitx_buff where the spi
 *                              command byte is located.
 *
 * @param [in]  PNTR_ID          SPI pointer ID to use as base address for writing 
 *                              the block of data.
 *
 * @param [in]  AddrIn          Address location at which the SPI write should 
 *                              start transfert in the PHY.
 *
 * @param [in]  nbBytes         Number of bytes to read or write to be performed.
 *                              If nbBytes is bigger than ADI_SPI_MEMORY_SIZE, 
 *                              the custom spi pntr0 will be used to transfert the data.
 *                              The shadow ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR]
 *                              and the PHY value (in the radio) will be restored 
 *                              to there original value.
 *
 * @param [in|out]  pDataIO     Pointer to the start of the Host data block.
 *                              On exit, pDataIO is pointing nbBytes away from
 *                              its original value.
 *
 * @param [in]  pDataRef        Pointer to the start of the Ref Host data block.
 *                              If set, the SPI readback data will be compared with the 
 *                              data pointed by pDataRef. This can be used to compare
 *                              data between Host and PHY directly within the readback.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__SPI_Block_Xfer(
    ADI_SPI_HANDLE                      hSPIDevice,
    uint32_t                            cmdOffset,
    ADI_ADF7030_1_RADIO_SPI_PNTR_TYPE   PNTR_ID,
    uint32_t                            Addr,
    uint32_t                            nbBytes,
    void *                              pDataIO,
    void *                              pDataRef
)
{
  
    /* Variable holding current value off full spi block (used when nbBytes > ADF7030_1_SPI_MAX_TRX_SIZE )*/
    uint32_t full_block_cnt = 0;

    /* Store the initial value of PNTR_CUSTOM0 */
    uint32_t copy_spi_pntr_custom0 = ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR];
    uint32_t copy_spi_pntr_ID = ADF7030_1_SPI_PNTR[PNTR_ID];

    /* Transfert loop */
    bool_t spi_transaction_finished = (nbBytes) ? false : true;

    while(spi_transaction_finished == false)
    {  
        uint32_t size;
      
        if( nbBytes > ADF7030_1_SPI_MAX_TRX_SIZE )
        {
            /* Full block transaction */
            size = ADF7030_1_SPI_MAX_TRX_SIZE;
        }
        else
        {
            /* Partial block transaction */
            size = nbBytes;
            spi_transaction_finished = true; 
        }
        
        if(spi_txbuf[cmdOffset] & ADF703x_SPI_MEM_LONG)
        {   
            /* Initiate the SPI word transfer */
            if(adf7030_1__SPI_word_block_transaction( &hSPIDevice,              //SPI handle
                                                      cmdOffset,                //SPI trx buffer offset
                                                      size >> 2,                //SPI nb_word to transfert
                                                      (uint32_t *)pDataIO,      //pointer to Host data to transfert
                                                      (uint32_t *)pDataRef      //pointer to Host Ref data for comparison
                                                    ) != SUCCESS)
            {
                return FAILURE;
            }
        }
        else
        {
            /* Initiate the SPI byte transfer */
            if(adf7030_1__SPI_byte_block_transaction( &hSPIDevice,              //SPI handle
                                                      cmdOffset,                //SPI trx buffer offset
                                                      size,                     //SPI nb_word to transfert
                                                      (uint8_t *)pDataIO,       //pointer to Host data to transfert
                                                      (uint8_t *)pDataRef       //pointer to Host Ref data for comparison
                                                    ) != SUCCESS)
            {
                return FAILURE;
            }

        }
        
        /* Increment pntrID by ADF7030_1_SPI_MAX_TRX_SIZE for next SPI transaction */
        if(spi_transaction_finished == false)
        {
            uint32_t tmp_SPI_Prolog;
            
            switch(spi_txbuf[cmdOffset] & 0x78)
            {
                case( ADF703x_SPI_MEM_WRITE | ADF703x_SPI_MEM_BLOCK |  ADF703x_SPI_MEM_ADDRESS | ADF703x_SPI_MEM_LONG ) :
                    //ADF703x_SPI_MEM_WRITE | ADF703x_SPI_MEM_BLOCK |  ADF703x_SPI_MEM_ADDRESS | ADF703x_SPI_MEM_LONG
              
                    /* Change the next Address in the spi_txbuf */
                    Addr += size;
                
                    /* Write the starting address in the spi_txbuf */
                    *(((uint32_t *)spi_txbuf) + 1) = __REV(Addr);
                    
                    break;

                case( ADF703x_SPI_MEM_READ | ADF703x_SPI_MEM_BLOCK |  ADF703x_SPI_MEM_ADDRESS | ADF703x_SPI_MEM_LONG ) :
                    //ADF703x_SPI_MEM_READ | ADF703x_SPI_MEM_BLOCK |  ADF703x_SPI_MEM_ADDRESS | ADF703x_SPI_MEM_LONG  
              
                    /* Change the next Address in the spi_txbuf */
                    Addr += size;
                
                    /* Write the starting address in the spi_txbuf */
                    *((uint16_t *)spi_txbuf + 1) = __REV16(*((uint16_t *)&Addr + 1));
                    *((uint16_t *)spi_txbuf + 2) = __REV16(*((uint16_t *)&Addr + 0));
                    
                    break;
              
                case( ADF703x_SPI_MEM_WRITE | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_POINTER | ADF703x_SPI_MEM_LONG ) :
                case( ADF703x_SPI_MEM_READ  | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_POINTER | ADF703x_SPI_MEM_LONG ) :
                case( ADF703x_SPI_MEM_WRITE | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_ADDRESS | ADF703x_SPI_MEM_SHORT ) :
                case( ADF703x_SPI_MEM_READ  | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_ADDRESS | ADF703x_SPI_MEM_SHORT ) :
                case( ADF703x_SPI_MEM_WRITE | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_POINTER | ADF703x_SPI_MEM_SHORT ) :
                case( ADF703x_SPI_MEM_READ  | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_POINTER | ADF703x_SPI_MEM_SHORT ) :
          
                    //ADF703x_SPI_MEM_WRITE | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_POINTER | ADF703x_SPI_MEM_LONG  | pntrID
                    //ADF703x_SPI_MEM_READ  | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_POINTER | ADF703x_SPI_MEM_LONG  | pntrID              
                    //ADF703x_SPI_MEM_WRITE | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_ADDRESS | ADF703x_SPI_MEM_SHORT | pntrID;
                    //ADF703x_SPI_MEM_READ  | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_ADDRESS | ADF703x_SPI_MEM_SHORT | pntrID;                             
                    //ADF703x_SPI_MEM_WRITE | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_POINTER | ADF703x_SPI_MEM_SHORT | pntrID;
                    //ADF703x_SPI_MEM_READ  | ADF703x_SPI_MEM_BLOCK | ADF703x_SPI_MEM_POINTER | ADF703x_SPI_MEM_SHORT | pntrID;
          
                    /* Change the Host copy of SPI pointer 0 address for next block of byte transfer */
                    ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] = copy_spi_pntr_ID + (size * ++full_block_cnt);
            
                    /* Save current SPI prolog */
                    tmp_SPI_Prolog = *(uint32_t *)spi_txbuf;
                    
                    /* Update the PHY Radio SPI pointer 0 */
                    adf7030_1__SPI_wr_word_b_p( hSPIDevice, PNTR_SETUP_ADDR, 1, &ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR]);
                    
                    /* Recall current SPI prolog */
                    *(uint32_t *)spi_txbuf = tmp_SPI_Prolog;
            
                    /* Update SPI command byte to use the SPI pointer 0 */
                    spi_txbuf[cmdOffset] &= 0xF8;
                    spi_txbuf[cmdOffset] |= PNTR_CUSTOM0_ADDR;
                    
                    break;
                
                default : return FAILURE;
            }
        }
        

        /* Increment pDataIO by size of the previous transaction */        
        if(pDataIO != NULL)
        {
            pDataIO = (uint8_t *)pDataIO + size;   
        }
        
        /* Increment the pDataRef by size of the previous transaction */ 
        if(pDataRef != NULL)
        {
            pDataRef = (uint8_t *)pDataRef + size;
        }
        
        /* Decrement the remaining nbBytes by the size of the previous transaction*/
        nbBytes -= size;
    }

    /* Restore the initial value of PNTR_CUSTOM0 if required */
    if(ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] != copy_spi_pntr_custom0)
    {
        /* Change the Host copy of SPI pointer 0 */
        ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR] = copy_spi_pntr_custom0;
        
        /* Update the PHY Radio SPI pointer 0 */
        adf7030_1__SPI_wr_word_b_p( hSPIDevice, PNTR_SETUP_ADDR, 1, &ADF7030_1_SPI_PNTR[PNTR_CUSTOM0_ADDR]);
    }
        
    return SUCCESS;    
}


/**
 * @brief       Generic SPI Block Read/Write a number of words(s) to the adf7030-1
 *                          
 * @note        Blocking SPI transfert without DMA. 
 *                          
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  offset          Byte location from where the spi frame starts in the spi_txbuf.
 *
 * @param [in]  nb_word         Number of Word(s) transfert.
 *
 * @param [in,out]  pDataIO     In TX, data is read from address pointed by pDataIO.
 *                              In RX, data is written back at address pointed by pDataIO.
 *
 * @param [in]  pDataRef        Pointer to the start of the Ref Host data block.
 *                              If set, the SPI readback data will be compared with the 
 *                              data pointed by pDataRef. This can be used to compare
 *                              data between Host and PHY directly within the readback.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__SPI_word_block_transaction(
    ADI_SPI_HANDLE *hSPIDevice,
    uint32_t offset,
    uint32_t nb_word,
    uint32_t * pDataIO,
    uint32_t * pDataRef
)
{
    /* SPI transceiver instance */
    ADI_SPI_TRANSCEIVER Transceiver;
    
    /* Check if we are in Address mode */
    uint8_t Addr_en = ((spi_txbuf[offset] & ADF703x_SPI_MEM_ADDRESS) >> 4);
    
    /* Initialize the transceiver */
    Transceiver.TransmitterBytes =      (4 - offset) + ((nb_word + Addr_en) << 2);
    Transceiver.ReceiverBytes    =      Transceiver.TransmitterBytes;
    Transceiver.pTransmitter     =      spi_txbuf + offset;    
    Transceiver.pReceiver        =      spi_rxbuf + offset;    
    Transceiver.nTxIncrement     =      1u;
    Transceiver.nRxIncrement     =      Transceiver.nTxIncrement;
    

    /* Set Block address to 2nd unit32_t */   
    uint32_t * pData = (((uint32_t *)spi_txbuf) + 1 + Addr_en);
    
    if(spi_txbuf[offset] & ADF703x_SPI_MEM_READ)
    {
    /* READ operation */
        
        /* Fill spi_txbuf with nb_word of NOPs */
        thumb_aligned_memset(pData, 0xFFFFFFFF, nb_word << 2);
    }
    else
    {
    /* WRITE operation */
        
        /* Copy nb_word from pDataIO to spi_rxbuf */  
        //thumb_aligned_memcpy(pData, pDataIO, size_alligned);
        for(uint16_t i = 0; i < nb_word; i++)
        {
            *pData++ = __REV(*pDataIO++); 
        }
    }
    
    /* Transmit the sequence */
    if(adi_spi_ReadWrite(*hSPIDevice, &Transceiver) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }
         
    if(spi_txbuf[offset] & ADF703x_SPI_MEM_READ)
    {
    /* READ operation */
      
        /* Set Block address to 2nd unit32_t */   
        pData = (((uint32_t *)spi_rxbuf) + 1 + Addr_en);
        
        if((pDataIO != NULL) && (pDataRef == NULL))
        {
            /* Copy nb_word from spi_rxbuf to pDataIO */  
            for(uint16_t i = 0; i < nb_word; i++)
            {
                *pDataIO++ = __REV(*pData++);
            }
        }
        else if((pDataIO != NULL) && (pDataRef != NULL))
        {
            uint32_t word;
            
            /* Copy up to nb_word from spi_rxbuf to pDataIO and Check versus pDataRef */
            for(uint16_t i = 0; i < nb_word; i++)
            {
                word = __REV(*pData++);
                if(*pDataRef++ != word)
                {
                    return FAILURE;
                }  
                *pDataIO++ = word;
            } 
        }
        else if((pDataIO == NULL) && (pDataRef != NULL))
        {
            /* Compare up to nb_word of spi_rxbuf versus pDataRef */
            for(uint16_t i = 0; i < nb_word; i++)
            {
                if(*pDataRef++ != __REV(*pData++))
                {
                    return FAILURE;
                }
            }
        }
        else
        {
            /* Should never end up here */
            return FAILURE;
        }
    }
    else
    {
    /* WRITE operation */
      
    }
       
    return SUCCESS;
}

/**
 * @brief       Generic SPI Block Read/Write a number of byte(s) to the adf7030-1
 *                          
 * @note        Blocking SPI transfert without DMA. 
 *                           
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  offset          Byte location from where the spi frame starts in the spi_txbuf.
 *
 * @param [in]  nb_byte         Number of Byte(s) transfert.
 *
 * @param [in,out]  pDataIO     In TX, data is read from address pointed by pDataIO.
 *                              In RX, data is written back at address pointed by pDataIO.
 *
 * @param [in]  pDataRef        Pointer to the start of the Ref Host data block.
 *                              If set, the SPI readback data will be compared with the 
 *                              data pointed by pDataRef. This can be used to compare
 *                              data between Host and PHY directly within the readback.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */
  
uint32_t adf7030_1__SPI_byte_block_transaction(
    ADI_SPI_HANDLE *hSPIDevice,
    uint32_t offset,
    uint32_t nb_bytes,
    uint8_t * pDataIO,
    uint8_t * pDataRef
)
{
    /* SPI transceiver instance */
    ADI_SPI_TRANSCEIVER Transceiver;
    
    /* Initialize the transceiver */
    Transceiver.TransmitterBytes =      (4 - offset) + nb_bytes;
    Transceiver.ReceiverBytes    =      Transceiver.TransmitterBytes;
    Transceiver.pTransmitter     =      spi_txbuf + offset;    
    Transceiver.pReceiver        =      spi_rxbuf + offset;    
    Transceiver.nTxIncrement     =      1u;
    Transceiver.nRxIncrement     =      Transceiver.nTxIncrement;
    

    /* Set Block address to 2nd unit32_t */   
    uint32_t * pData = (((uint32_t *)spi_txbuf) + 1);
 
    /* Set the size of alligned full words (32bits) */
    uint32_t size_alligned = (nb_bytes >> 2) << 2;
   
    /* Set the number of remaining bytes (8bits) */
    uint32_t rem_byte_num_xfrs = nb_bytes & 0x3;
    
    if(spi_txbuf[offset] & ADF703x_SPI_MEM_READ)
    {
    /* READ operation */
        
        /* Fill spi_txbuf with nb_word of NOPs */
        if(size_alligned) thumb_aligned_memset(pData, 0xFFFFFFFF, size_alligned);
    
        /* Fill spi_txbuf with rem_byte_num_xfrs of NOPs  */   
        if(rem_byte_num_xfrs)
        {
            uint8_t *pRemData = (uint8_t *)pData + size_alligned;
                  
            for(uint32_t i = 0; i < rem_byte_num_xfrs; i++)
            {
                *pRemData++ = 0xFF;
            }
        }
    }
    else
    {
    /* WRITE operation */
        
        /* Copy nb_word from pDataIO to spi_rxbuf */  
        if(size_alligned) thumb_aligned_memcpy(pData, pDataIO, size_alligned);
         
        /* Copy rem_byte_num_xfrs from pDataIO to spi_txbuf */ 
        if(rem_byte_num_xfrs)
        {
            uint8_t *pRemData = (uint8_t *)pData + size_alligned;
            pDataIO += size_alligned;
                  
            for(uint32_t i = 0; i < rem_byte_num_xfrs; i++)
            {
                *pRemData++ = *pDataIO++;
            }
        }    
    }
    
    /* Transmit the sequence */
    if(adi_spi_ReadWrite(*hSPIDevice, &Transceiver) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }
         
    if(spi_txbuf[offset] & ADF703x_SPI_MEM_READ)
    {
    /* READ operation */
      
        /* Set Block address to 2nd unit32_t */   
        pData = (((uint32_t *)spi_rxbuf) + 1);
    
        if((pDataIO != NULL) && (pDataRef == NULL))
        {
            /* Copy nb_word from spi_rxbuf to pDataIO */  
            if(size_alligned) thumb_aligned_memcpy(pDataIO, pData, size_alligned);
    
            /* Copy rem_byte_num_xfrs from spi_rxbuf to pDataIO */   
            if(rem_byte_num_xfrs)
            {
                uint8_t *pRemData = (uint8_t *)pData + size_alligned;
                pDataIO += size_alligned;
        
                for(uint32_t i = 0; i < rem_byte_num_xfrs; i++)
                {
                    *pDataIO++ = *pRemData++;
                }
            }        
        }
        else if((pDataIO != NULL) && (pDataRef != NULL))
        {
            uint8_t byte;
            uint8_t * pDataByte = (uint8_t *)pData;
            
            /* Copy up to nb_bytes from spi_rxbuf to pDataIO and Check versus pDataRef */
            for(uint16_t i = 0; i < nb_bytes; i++)
            {
                byte = *pDataByte++;
                if(*pDataRef++ != byte)
                {
                    return FAILURE;
                }  
                *pDataIO++ = byte;
            } 
        }
        else if((pDataIO == NULL) && (pDataRef != NULL))
        {
            uint8_t * pDataByte = (uint8_t *)pData;
            
            /* Compare up to nb_bytes of spi_rxbuf versus pDataRef */
            for(uint16_t i = 0; i < nb_bytes; i++)
            {
                if(*pDataRef++ != *pDataByte++)
                {
                    return FAILURE;
                }
            }
        }
        else
        {
            /* Should never end up here */
            return FAILURE;
        }
    }
    else
    {
    /* WRITE operation */
      
    }
    
    
    return SUCCESS;

}  

/**
 * @brief       Generic SPI Random Read/Write a number of words(s) to the adf7030-1
 *                          
 * @note        Blocking SPI transfert without DMA. 
 *                          
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  offset          Byte location from where the spi frame starts in the spi_txbuf.
 *
 * @param [in]  nb_word         Number of Word(s) transfert.
 *
 * @param [in,out]  pDataIO     In TX, data is read from address pointed by pDataIO.
 *                              In RX, data is written back at address pointed by pDataIO.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__SPI_word_random_transaction(
    ADI_SPI_HANDLE *hSPIDevice,
    uint32_t offset,
    uint32_t nb_word,
    uint32_t *pRemoteAddrIO,
    uint32_t *pHostDataIO,
    bool pHostDataIO_isArray
)
{
    /* SPI transceiver instance */
    ADI_SPI_TRANSCEIVER Transceiver;
    
    /* Check if we are in Writing mode */
    uint8_t Write_en = ((~(spi_txbuf[offset]) & ADF703x_SPI_MEM_READ) >> 6);

    
    /* Initialize the transceiver */
    Transceiver.TransmitterBytes =      (4 - offset) + (nb_word << (2 + Write_en));
    Transceiver.ReceiverBytes    =      Transceiver.TransmitterBytes;
    Transceiver.pTransmitter     =      spi_txbuf + offset;    
    Transceiver.pReceiver        =      spi_rxbuf + offset;    
    Transceiver.nTxIncrement     =      1u;
    Transceiver.nRxIncrement     =      Transceiver.nTxIncrement;
    
    /* Set Block address to 2nd unit32_t */   
    uint32_t * pData = (((uint32_t *)spi_txbuf) + 1);
    
    if(Write_en)
    {
    /* WRITE operation */
        
        /* Copy nb_word from alternating pRemoteAddrIO & pHostDataIO to spi_txbuf */  
        for(uint16_t i = 0; i < nb_word; i++)
        {
            uint32_t data = *(uint32_t *)pHostDataIO++;

            *pData++ = __REV(*pRemoteAddrIO++);
            *pData++ = __REV((pHostDataIO_isArray == true) ? *(uint32_t *)data : data);   
        }
    }
    else
    {
    /* READ operation */
               
        /* Fill spi_txbuf with nb_word of pRemoteAddrIO */
        for(uint16_t i = 0; i < nb_word; i++)
        {
            *pData++ = __REV(*pRemoteAddrIO++);
        }
        
        /* Add two extra bytes for missaligned word read on the spi */
        Transceiver.TransmitterBytes += 2;
        Transceiver.ReceiverBytes    =      Transceiver.TransmitterBytes;
 
        /* Pad the spi_txbuf with 2bytes off extra SPI_NOP */
        *(uint16_t *)pData = 0xFFFF;
    }
    
    /* Transmit the sequence */
    if(adi_spi_ReadWrite(*hSPIDevice, &Transceiver) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }
         
    if(!Write_en)
    {
    /* READ operation */
      
        /* Set Block address to 2nd unit32_t */   
        uint16_t *pDataOut = (((uint16_t *)spi_rxbuf) + 4);
    
        /* Copy nb_word from spi_rxbuf to pDataIO */  
        for(uint16_t i = 0; i < ((nb_word << 1) + 1); i++)
        {
            uint32_t data;
           
            data  = __REV16(*pDataOut++) << 16;
            data |= __REV16(*pDataOut++);
            
            if(pHostDataIO_isArray == true)
            {
                *pHostDataIO++ = data;
            }
            else
            {
                uint32_t * pHostData = pHostDataIO++;                
                *pHostData = data;
            }  
        } 
    }
       
    return SUCCESS;  
}

/**
 * @brief       Generic SPI Random Read/Write a number of byte(s) to the adf7030-1
 *                          
 * @note        Blocking SPI transfert without DMA. 
 *                           
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  offset          Byte location from where the spi frame starts in the spi_txbuf.
 *
 * @param [in]  nb_byte         Number of Byte(s) transfert.
 *
 * @param [in,out]  pDataIO     In TX, data is read from address pointed by pDataIO.
 *                              In RX, data is written back at address pointed by pDataIO.
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */
/*  
uint32_t adf7030_1__SPI_byte_random_transaction(
    ADI_SPI_HANDLE *hSPIDevice,
    uint32_t offset,
    uint32_t nb_bytes,
    uint8_t * pDataIO
)
{
  
}
*/

#endif /* _ADF7030_1__SPI_C_ */

/** @} */ /* End of group adf7030-1__spi SPI Interface layer */
/** @} */ /* End of group adf7030-1 adf7030-1 Driver */
