/*********************************************************************************

Copyright(c) 2016 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file     adf7030-1__state.c
*
* @brief    adf7030-1 State Machine Control Functions.
*               - Sending PHY command.
*               - State transitions
*
* @details  ...
*           ...
*           ...
*
* @note     Current generic "adi_spi" driver does not implement continuous polling
*           mechanism and is extremely slow. adf7030-1 drivers will get performance 
*           boost once new SPI rf driver is available.
*/

/** \addtogroup adf7030-1 adf7030-1 Driver
 *  @{
 */

/** \addtogroup adf7030-1__state PHY State Machine Interface
 *  @{
 */
#ifndef _ADF7030_1__STATE_C_
/*! \cond PRIVATE */
#define _ADF7030_1__STATE_C_

#include <stddef.h>
#include <string.h>
#include <assert.h>
#include "system.h"
#include "common.h"

#include <drivers/spi/adi_spi.h>
//#include <devices/rf/adf703x/adi_adf7030-1.h>
//#include <devices/rf/adf703x/adi_adf7030-1_reg.h>
#include <devices/rf/adf703x/adf7030-1__spi.h>
#include <devices/rf/adf703x/adf7030-1__mem.h>
#include <devices/rf/adf703x/adf7030-1__state.h>


extern uint8_t      spi_txbuf[ADF7030_1_SPI_BUFFER_SIZE];
extern uint8_t      spi_rxbuf[ADF7030_1_SPI_BUFFER_SIZE];





#ifdef __ICCARM__
/*
* IAR MISRA C 2004 error suppressions.
*
* Pm140 (rule 11.3): a cast should not be performed between a pointer type and an integral type
*   The rule makes an exception for memory-mapped register accesses.
*
* Pm073 (rule 14.7): a function should have a single point of exit.
* Pm143 (rule 14.7): a function should have a single point of exit at the end of the function.
*   Multiple returns are used for error handling.
*/
#pragma diag_suppress=Pm073,Pm143
#endif /* __ICCARM__ */

/*! \endcond */




/**
 * @brief       Generic Radio PHY Command Transfers
 *
 * @param [in]  hSPIDevice      Handle to Glue SPI peripheral device used to 
 *                              communicate with the adf7030-1.
 *
 * @param [in]  PHY_CMD         PHY Command from ADI_ADF7030_1_RADIO_STATE.
 *
 * @note                        This function does not use SPI PHY Radio polling
 *                              mechanism. It need to be reimplemented once new
 *                              SPI drivers is ready.
 *                              The "Wait for cmd_ready bit" is not correct
 *
 * @return      Status
 *  - #SUCCESS                  If the transfert was succesfull to the adf7030-1.
 *  - #FAILURE                  [D] If the transfert failed.
 */

uint32_t adf7030_1__XferCmd(
    ADI_SPI_HANDLE hSPIDevice,
    ADI_ADF7030_1_RADIO_STATE PHY_CMD
)
{   
    /* SPI transceiver instance */
    ADI_SPI_TRANSCEIVER Transceiver;
    
    /* Initialize the transceiver */
    Transceiver.TransmitterBytes =      1;
    Transceiver.ReceiverBytes    =      1;
    Transceiver.pTransmitter     =      spi_txbuf;    
    Transceiver.pReceiver        =      spi_rxbuf;    
    Transceiver.nTxIncrement     =      0u;
    Transceiver.nRxIncrement     =      Transceiver.nTxIncrement;
    
    /* Set the PHY Command into the spi_txbuf */
    spi_txbuf[0] = (uint8_t)(RADIO_CMD | PHY_CMD);
    
    /* Transmit the sequence */
    if(adi_spi_ReadWrite(hSPIDevice, &Transceiver) != ADI_SPI_SUCCESS)
    {
        return FAILURE;
    }
        
    /* Wait until "cmd_ready" bit is set */
    ADI_ADF7030_1_RADIO_SPI_STATUS_TYPE PHY_STATUS;
    do
    {
       //PHY_STATUS.VALUE = adf7030_1__SPI_GetMem8(hSPIDevice, SPI_HOST_STATUS_Addr);
       adf7030_1__SPI_rd_byte_b_a( hSPIDevice, PNTR_SETUP_ADDR, 0x14, 1, &PHY_STATUS.VALUE);
    }while((PHY_STATUS.CMD_READY == 0) && (PHY_STATUS.FW_STATUS == TRANSITION_STATUS_TRANSITION));

    return SUCCESS;
}

        
#ifdef ADI_SPI_v2
    ADI_SPI_HANDLE *hDevice = hSPIDevice; 
    ADI_SPI_RESULT eResult;
    hDevice->bBlockingMode = true;
    //eResult = adi_spi_MasterTransfer(hDevice,pXfr);
    do{
    
    ADI_SPI_RESULT result = ADI_SPI_SUCCESS;
    volatile uint16_t nStatus;

#ifdef ADI_DEBUG
    if (ADI_SPI_VALIDATE_HANDLE(hDevice))
    {
        eResult = ADI_SPI_INVALID_HANDLE;
        return FAILURE;
    }

    if (ADI_SPI_STATE_MASTER != hDevice->eDevState)
    {
        eResult = ADI_SPI_ERR_NOT_INITIALIZED;
        return FAILURE;
    }

    if ((NULL == pXfr->pTransmitter) && (NULL == pXfr->pReceiver))
    {
        eResult = ADI_SPI_INVALID_POINTER;
        return FAILURE;
    }

    /* yell about odd byte counts in debug mode for  DMA requests */
    if ((hDevice->bDmaMode== true) && ((pXfr->TransmitterBytes&1u)!=0u))
    {
        eResult = ADI_SPI_INVALID_PARAM;
        return FAILURE;
    }

    /* yell about odd byte counts in debug mode for  DMA requests */
    if ((hDevice->bDmaMode== true) && ((pXfr->ReceiverBytes&1u)!=0u))
    {
        eResult = ADI_SPI_INVALID_PARAM;
        return FAILURE;
    }
    /* Return error if the RX buffer is not null and count is equal to zero or vice versa.*/
    if (((pXfr->pReceiver != NULL) && (pXfr->ReceiverBytes == 0u)) || ((pXfr->pReceiver == NULL) && ((pXfr->ReceiverBytes > 0u))))
    {
        eResult = ADI_SPI_INVALID_PARAM;
        return FAILURE;
    }
    
    /* Return error if the Tx buffer is not null and count is equal to zero or vice versa.*/
    if (((pXfr->pTransmitter != NULL) && (pXfr->TransmitterBytes == 0u)) || ((pXfr->pTransmitter == NULL) && (pXfr->TransmitterBytes > 0u)))
    {
        eResult = ADI_SPI_INVALID_PARAM;
        return FAILURE;
    }

	/* DMA count register is only 8 bits, so block size is limited to 255 */
    if ((hDevice->bDmaMode==true) && (pXfr->TransmitterBytes != 0u) &&(((uint32_t)pXfr->pTransmitter&0x1u) !=0u ) )
    {
        eResult = ADI_SPI_INVALID_PARAM;
        return FAILURE;
    }

    /* yell about (non-zero) odd byte counts in debug mode for DMA requests */
    if (   (hDevice->bDmaMode == true)
        && ((pXfr->ReceiverBytes != 0u) && (((uint32_t)pXfr->pReceiver & 0x1u) != 0u) ))
    {
        eResult = ADI_SPI_INVALID_PARAM;
        return FAILURE;
    }


     /* DMA count register is only 8 bits, so block size is limited to 255 */
    if (   (hDevice->bDmaMode==true)
        && (   ((pXfr->ReceiverBytes    & (~(uint32_t)BITM_SPI_CNT_VALUE)) !=0u)
            || ((pXfr->TransmitterBytes & (~(uint32_t)BITM_SPI_CNT_VALUE)) !=0u)))
    {
        eResult = ADI_SPI_INVALID_PARAM;
        return FAILURE;
    }

    if (((pXfr->ReceiverBytes == 0u) && (pXfr->TransmitterBytes == 0u)))
    {
        eResult = ADI_SPI_INVALID_PARAM;
        return FAILURE;
    }
#endif /* ADI_DEBUG */

    /* disable interrupts during manual prologue phase */
    ADI_DISABLE_INT(hDevice->pDevInfo->eIRQn);
    hDevice->pSpi->CTL &= (uint16_t)~(BITM_SPI_CTL_TIM | BITM_SPI_CTL_RFLUSH | BITM_SPI_CTL_TFLUSH);
    hDevice->pSpi->IEN |= (BITM_SPI_IEN_XFRDONE);
    /* initialize the data phase */
    hDevice->pTxBuffer    = spi_txbuf;
    hDevice->pRxBuffer    = spi_rxbuf;
    hDevice->TxRemaining  = 1;
    hDevice->RxRemaining  = 1;
    hDevice->TxIncrement  = 1;
    hDevice->RxIncrement  = 1;

    /* clear any residual */
    nStatus = hDevice->pSpi->STAT;
    hDevice->pSpi->STAT = nStatus;

    hDevice->pSpi->CTL |= ( BITM_SPI_CTL_MASEN) ;

    /* assert chip select */
    if (ADI_SPI_SUCCESS != (result = AssertChipSelect(hDevice, true)))
    {
        return result;
    }

    /* initialize DMA descriptors and channel enables through the function pointer table */
    if (ADI_SPI_SUCCESS != (result = hDevice->FunctionTable.pInitDescriptorsFcn(hDevice)))
    {
        AssertChipSelect(hDevice, false);
	return result;
    }

    /* block as required on both channels */
    if (hDevice->bBlockingMode == true)
    {
#if (ADI_CFG_ENABLE_RTOS_SUPPORT == 1)
	PEND_EVENT(hDevice,ADI_SPI_SEMAPHORE_FAILED);
#else
#if (ADI_CFG_ENTER_LOW_PWR_MODE_SUPPORT == 1)
        /* Instead of busy-waiting, we enter core sleep mode while waiting. Any
   		 * interrupt will also bring the processor out of core sleep and cause this call to return.
		*/

		PEND_EVENT(hDevice,ADI_SPI_FAILED);
#else
	   {
		   bool_t bComplete;
		   adi_spi_MasterComplete(hDevice,&bComplete);
		   while (bComplete == false)
		    {
				adi_spi_MasterComplete(hDevice,&bComplete);
            }
	    }
#endif
#endif /* ADI_CFG_ENABLE_RTOS_SUPPORT  */
	}
    return result;
    }while
    
    
    
    
    hDevice->bBlockingMode = false;
#endif /* ADI_SPI_v2 */
    


#endif /* _ADF7030_1__PATCH_C_ */

/** @} */ /* End of group adf7030-1__state PHY State Machine Interface */
/** @} */ /* End of group adf7030-1 adf7030-1 Driver */
