/*********************************************************************************

Copyright(c) 2016 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file     adf7030-1__timer.c
*
* @brief    Generic Timer functions between Glue and Radio.
*
* @details  ...
*           ...
*           ...
*/

/** \addtogroup adf7030-1 adf7030-1 Driver
 *  @{
 */

/** \addtogroup adf7030-1__timer Timers Interface layer
 *  @{
 */

#ifndef _ADF7030_1__TIMER_C_
/*! \cond PRIVATE */
#define _ADF7030_1__TIMER_C_

#include <stddef.h>
#include <string.h>
#include <assert.h>
#include "system.h"
#include "common.h"

#include <services/tmr/adi_tmr.h>
#include <services/gpio/adi_gpio.h>
//#include <devices/rf/adf703x/adi_adf7030-1.h>
//#include <devices/rf/adf703x/adi_adf7030-1_reg.h>
#include <devices/rf/adf703x/adf7030-1__timer.h>

#ifdef __ICCARM__
/*
* IAR MISRA C 2004 error suppressions.
*
* Pm140 (rule 11.3): a cast should not be performed between a pointer type and an integral type
*   The rule makes an exception for memory-mapped register accesses.
*
* Pm073 (rule 14.7): a function should have a single point of exit.
* Pm143 (rule 14.7): a function should have a single point of exit at the end of the function.
*   Multiple returns are used for error handling.
*/
#pragma diag_suppress=Pm073,Pm143
#endif /* __ICCARM__ */

/*! \endcond */

#define ADF7030_1_TIMER_TICK 2
#define ADF7030_1_TIMER_FAST 0

#define SYS_CLK_FREQ 26000000
#define ADF7030_1_TIMER_1MS_VAL (SYS_CLK_FREQ / 1000)
#define ADF7030_1_TIMER_1US_VAL (ADF7030_1_TIMER_1MS_VAL / 1000)

/* Memory required for TIMER driver */
static uint8_t aDeviceMemory0[ADI_TMR_MEMORY_SIZE];
static uint8_t aDeviceMemory2[ADI_TMR_MEMORY_SIZE];

/* Timers device handles */
static ADI_TMR_HANDLE   hTMR0Device;
static ADI_TMR_HANDLE   hTMR2Device;

/* Global Timestamp */
ADI_ADF7030_1_TIMESTAMP adf7030_1_time;
ADI_ADF7030_1_TIMESTAMP adf7030_1_time_capture;

volatile bool_t bWait_uS_Flag;

/* Initializes TIMER device */
uint32_t adf7030_1__TIMER_Init
(
    void
)
{
    /* Configure Timer2 as a background Global Time Stamp timer */  
    if(adi_tmr_Open(ADF7030_1_TIMER_TICK,
                    aDeviceMemory2,
                    ADI_TMR_MEMORY_SIZE,
                    &hTMR2Device) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }

    /* Set prescalar */
    if(adi_tmr_SetPrescaler(hTMR2Device, ADI_GPT_PRESCALER_1) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Set clock source */
    if(adi_tmr_SetClockSource(hTMR2Device, ADI_TMR_CLOCK_PCLK) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Set to run in periodic mode */
    if(adi_tmr_SetRunMode(hTMR2Device, ADI_TMR_PERIODIC_MODE) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Set to count down from Load value */
    if(adi_tmr_SetCountMode(hTMR2Device, ADI_TMR_COUNT_DOWN) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Enable automatic reloading (optional???) */
    if(adi_tmr_EnableReloading(hTMR2Device, true) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Set Load Value to give us a timeout every 1ms */
    if(adi_tmr_SetLoadValue(hTMR2Device, ADF7030_1_TIMER_1MS_VAL / ERROR_COMPENSATION_DIV) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    } 
    
    /* Register the Timer2 Callback function */
    if(adi_tmr_RegisterCallback(hTMR2Device, GPTimer2Callback, hTMR2Device) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }

    /* Configure capture event on external Interrupt from the adf7030-1 radio SPI */
    if(adi_tmr_SetEventToCapture(hTMR2Device, ADI_TMR2_CAPTURE_EVENT_EXT_INT_0) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Enable the capture event */
    if(adi_tmr_EnableEventCapture(hTMR2Device, true) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Clear the time stamp global variable */
    adf7030_1_time.ms = 0;
    adf7030_1_time.ticks = 0;
    
    /* Start up the Timer2 */
    if(adi_tmr_Enable(hTMR2Device, true) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Configure Timer0 as general purpose (us resolution) timer for wait and timeout  */ 
    if(adi_tmr_Open(ADF7030_1_TIMER_FAST,
                    aDeviceMemory0,
                    ADI_TMR_MEMORY_SIZE,
                    &hTMR0Device) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }

    /* Set prescalar */
    if(adi_tmr_SetPrescaler(hTMR0Device, ADI_GPT_PRESCALER_1) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Set clock source */
    if(adi_tmr_SetClockSource(hTMR0Device, ADI_TMR_CLOCK_PCLK) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Set to run in periodic mode */
    if(adi_tmr_SetRunMode(hTMR0Device, ADI_TMR_PERIODIC_MODE) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Set to count down from Load value */
    if(adi_tmr_SetCountMode(hTMR0Device, ADI_TMR_COUNT_DOWN) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Disable automatic reloading */
    if(adi_tmr_EnableReloading(hTMR0Device, false) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
      
    return SUCCESS;
}

/* Uninitializes TIMER device */
uint32_t adf7030_1__TIMER_UnInit
(
    void
)
{
    if(adi_tmr_EnableEventCapture(hTMR2Device, false) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    if(adi_tmr_Enable(hTMR2Device, false) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }

    if(adi_tmr_Close(hTMR2Device) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
     
    if(adi_tmr_Enable(hTMR0Device, false) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }

    if(adi_tmr_Close(hTMR0Device) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }

    
    return SUCCESS;
}

/* Clear the Global Time Count */
uint32_t adf7030_1__TIMER_ClrTime
(
    ADI_ADF7030_1_TIMESTAMP * pTime
)
{
    pTime->ms = 0; 
    pTime->ticks = 0;
    
    return SUCCESS;
}


/* Capture the current Time Count */
uint32_t adf7030_1__TIMER_GetTime
(
    ADI_ADF7030_1_TIMESTAMP * pTime
)
{
    pTime->ms = adf7030_1_time.ms;
    if(adi_GPT_GetCurrentValue(hTMR2Device, &pTime->ticks) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    return SUCCESS;
}


uint32_t adf7030_1__TIMER_Start_uS
(
    const uint16_t u_sec,
    ADI_CALLBACK const pfCallback
)
{
    if(u_sec > TOTAL_TMR_ERROR)
    {
        /* Calculate the Load value for the desired uS delay */
        //uint32_t Val_uS = ADF7030_1_TIMER_1US_VAL * ((u_sec > TOTAL_TMR_ERROR) ? (u_sec - TOTAL_TMR_ERROR) : ERROR_COMPENSATION_DIV);
        uint32_t Val_uS = ADF7030_1_TIMER_1US_VAL * (u_sec - TOTAL_TMR_ERROR);
           
        /* Saturates at 2530 us max */
        if(Val_uS >> 16) Val_uS = 65520;
        
        /* Set Load Value to give u_sec uS a delay */
        if(adi_tmr_SetLoadValue(hTMR0Device, ((uint16_t)Val_uS) / ERROR_COMPENSATION_DIV) != ADI_TMR_SUCCESS)
        {
            return FAILURE;
        } 
        
        /* Register custom Timer0 Callback function */
        if(adi_tmr_RegisterCallback(hTMR0Device, pfCallback, hTMR0Device) != ADI_TMR_SUCCESS)
        {
            return FAILURE;
        }
        
        bWait_uS_Flag = true;
        
        adi_gpio_SetHigh(ADI_GPIO_PORT0, ADI_GPIO_PIN_12);

        /* Start up the Timer0 */
        if(adi_tmr_Enable(hTMR0Device, true) != ADI_TMR_SUCCESS)
        {
            return FAILURE;
        }
    }
    else
    {
        adi_gpio_SetHigh(ADI_GPIO_PORT0, ADI_GPIO_PIN_12);
        
        adf7030_1__TIMER_Active_Delay_uS(u_sec);
        
        bWait_uS_Flag = false;
        adf7030_1__TIMER_GetTime(&adf7030_1_time_capture);
        
        adi_gpio_SetLow(ADI_GPIO_PORT0, ADI_GPIO_PIN_12);
        
    }
    return SUCCESS;
}

uint32_t adf7030_1__TIMER_Stop_uS
(
    void
)
{   
    /* Disable the Timer0 */
    if(adi_tmr_Enable(hTMR0Device, false) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    /* Unregister Timer0 Callback function */
    if(adi_tmr_RegisterCallback(hTMR0Device, NULL, hTMR0Device) != ADI_TMR_SUCCESS)
    {
        return FAILURE;
    }
    
    bWait_uS_Flag = false;

    return SUCCESS;
}

uint32_t adf7030_1__TIMER_Wait_uS
(
    const uint16_t u_sec,
    ADI_CALLBACK const pfCallback
)
{
    if(adf7030_1__TIMER_Start_uS(u_sec, pfCallback) != SUCCESS)
    {
        return FAILURE;
    }
    
    while(bWait_uS_Flag == true);
    
    if(adf7030_1__TIMER_Stop_uS() != SUCCESS)
    {
        return FAILURE;
    }
    
    return SUCCESS;
}

/// @brief Active wait using NOPs
/// @description This function loops using NOPs to give short delays
/// @param[in]  delay_us    Delay in microseconds
uint32_t adf7030_1__TIMER_Active_Delay_uS
(
    const uint16_t delay_us
)
{
    // The loop below has 8 ticks per pass in the Cortex M0
    #define INSTRUCTIONS_PER_LOOP 8
    
    // exit immediately if no delay
    if (delay_us) {
        // The number of loops per us multiplied by the number of us, all divided by the number of system ticks per loop
        const uint32_t cycles_per_loop = INSTRUCTIONS_PER_LOOP;
        const uint32_t usec_div        = ADF7030_1_TIMER_1US_VAL;
        const uint32_t loops           = (delay_us * usec_div) / cycles_per_loop;
        
        for (uint32_t i = 0; i < loops; i++) {
          //\todo: compiler pragma to stop optimisation of NOPs
            __NOP();
            __NOP();
            __NOP();
            __NOP();
        }
    }
        
    //Always return ok for now
    return SUCCESS;
}


/* Default Callback event parsing for TIMER0 device (GP high resolution timer) */
void GPTimer0Callback
(
    void *pCBParam,
    uint32_t Event,
    void *pArg
)
{
  switch(Event)
  {
      case ADI_TMR_EVENT_TIMEOUT:      
        bWait_uS_Flag = false;
        adf7030_1__TIMER_GetTime(&adf7030_1_time_capture);
        break;
    default:
        break;
  }
  adi_gpio_SetLow(ADI_GPIO_PORT0, ADI_GPIO_PIN_12);
}


/* Default Callback event parsing for TIMER2 device (Timestamp) */
void GPTimer2Callback
(
    void *pCBParam,
    uint32_t Event,
    void *pArg
)
{
  adi_gpio_SetHigh(ADI_GPIO_PORT2, ADI_GPIO_PIN_0);
  switch(Event)
  {
      case ADI_TMR_EVENT_TIMEOUT:
        adf7030_1_time.ms += 1; 
        adf7030_1_time.ticks = 0;
        break;
      case ADI_TMR_EVENT_CAPTURED:
        adf7030_1_time_capture.ms = adf7030_1_time.ms;
        adi_tmr_GetCapturedValue(pCBParam, &adf7030_1_time_capture.ticks);
        break;
     default:
        break;
  }  
  adi_gpio_SetLow(ADI_GPIO_PORT2, ADI_GPIO_PIN_0);
}


#endif /* _ADF7030_1__TIMER_C_ */

/** @} */ /* End of group adf7030-1__timer Timers Interface layer */
/** @} */ /* End of group adf7030-1 adf7030-1 Driver */
